//
//  AppDelegate.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 04/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework
import PKHUD

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?
    
    var clientId = <#String#>
    var clientSecret =  <#String#>
    var urlScheme =  <#String#>  // For example "sdkdemo.utility://"
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        
        window = UIWindow(frame: UIScreen.main.bounds)
        window?.rootViewController = LaunchController()
        window?.makeKeyAndVisible()
        
        configureApp()
        showInitialController()

        return true
    }
    
    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        
        let sourceApplication = options[UIApplication.OpenURLOptionsKey.sourceApplication] as? String
        let annotation = options[UIApplication.OpenURLOptionsKey.annotation]
        if ZohoAuth.handleURL(url, sourceApplication: sourceApplication, annotation: annotation) {
            return true
        }
        return false
    }
}

extension AppDelegate {
    
    func configureApp() {
        let scope = ["ZohoCreator.meta.READ","ZohoCreator.data.READ","ZohoCreator.meta.CREATE","ZohoCreator.data.CREATE"]
        ZohoAuth.initWithClientID(clientId, clientSecret: clientSecret, scope: scope, urlScheme: urlScheme, mainWindow: window, accountsURL: "https://accounts.zoho.com")
        Creator.configure(delegate: self)
    }
    
    func showInitialController() {
        ZohoCreatorAPIHandler.fetchMyUserInfo(shouldCacheResponse: false) { (result) in
            switch result {
            case .failure( _):
                self.window?.rootViewController = LaunchController()
            case .success(_):
                if let appOwnerName = AppConstants.appOwner, let appLinkName = AppConstants.appLinkName {
                    let sectionVC = SectionListingViewController(withApp: Application(appOwnerName: appOwnerName, appLinkName: appLinkName, appName: "Utility"))
                    let  navContoller = UINavigationController.init(rootViewController: sectionVC)
                    self.window?.rootViewController = navContoller
                }
            }
        }
    }
    
}


extension AppDelegate : ZCCoreFrameworkDelegate {
    
    func userName() -> String? {
        return nil
    }
    
    func loginUserId() -> String? {
        return nil
    }
    
    func oAuthToken(with completion: @escaping AccessTokenCompletion) {
        ZohoAuth.getOauth2Token { (token, error) in
            if token == nil {
                ZohoAuth.presentZohoSign(in: { (token, error) in
                    completion(token,error)
                })
            }
            else {
                completion(token,error)
            }
        }
    }
    
}

class LaunchController: UIViewController {
    override func viewDidLoad() {
        super.viewDidLoad()
        if let launchView = UIStoryboard(name: "LaunchScreen", bundle: nil).instantiateInitialViewController()?.view {
            view.addSubview(launchView)
            launchView.translatesAutoresizingMaskIntoConstraints = false
            launchView.leftAnchor.constraint(equalTo: view.leftAnchor).isActive = true
            launchView.rightAnchor.constraint(equalTo: view.rightAnchor).isActive = true
            launchView.topAnchor.constraint(equalTo: view.topAnchor).isActive = true
            launchView.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        }
    }
}

extension UIViewController {
    func showActionLoading(show:Bool){
        if show {
            PKHUD.sharedHUD.contentView = PKHUDProgressView()
            PKHUD.sharedHUD.show(onView: view)
        } else {
            PKHUD.sharedHUD.hide()
        }
    }
    
    func showMessage(message:String) {
        let alert = UIAlertController(title: nil, message: message, preferredStyle: .alert)
        self.present(alert, animated: true, completion:nil)
        alert.addAction(UIAlertAction(title: "Done", style: .cancel, handler: nil))
    }
    
    static func topViewController() -> UIViewController? {
        var topViewController = UIApplication.shared.keyWindow?.rootViewController
        while topViewController?.presentedViewController != nil {
            topViewController = topViewController?.presentedViewController
        }
        return topViewController
    }
    
}
