//
//  CanteenCell.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 06/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import Foundation
import ZCCoreFramework

class CanteenCell: UICollectionViewCell {
    
    var selectedCell = false
    var choice: Choice?
    var isSelectedCell : Bool? {
        didSet
        {
            self.selectedCell = !selectedCell
            updateCellSelection()
        }
    }
    
    lazy var selectedCellImageView : UIImageView = {
            let imageView = UIImageView()
            imageView.image = UIImage.init(named: "done") ?? nil
            imageView.contentMode = .scaleAspectFit
            imageView.translatesAutoresizingMaskIntoConstraints = false
            let xOrigin:CGFloat = self.frame.size.width/2 - (imageView.image?.size.width)!
            
            self.addSubview(imageView)
            
            NSLayoutConstraint.activate([
                imageView.centerXAnchor.constraint(equalTo: self.centerXAnchor, constant: self.frame.size.width/2 - 20.00),
                imageView.bottomAnchor.constraint(equalTo: self.bottomAnchor, constant: -15),
                ])
            imageView.isHidden = true
            return imageView
    }()
    
    lazy var titleLable : UILabel = {
            let titleLable = UILabel()
            titleLable.translatesAutoresizingMaskIntoConstraints = false
            contentView.addSubview(titleLable)
            titleLable.font = UIFont.systemFont(ofSize: 16, weight: UIFont.Weight.semibold)
            titleLable.textColor  = UIColor.init(red: 0/255.0, green: 0/255.0, blue: 0/255.0, alpha: 1/1.0)
            
            let titleLableWidth : CGFloat = self.frame.size.width
            let titleLableHeight : CGFloat = 22
            
            self.addConstraint(NSLayoutConstraint(item: titleLable, attribute: .left, relatedBy: .equal, toItem: self, attribute: .left, multiplier: 1.0, constant: 15))
            self.addConstraint(NSLayoutConstraint(item: titleLable, attribute: .right, relatedBy: .equal, toItem: self, attribute: .right, multiplier: 1.0, constant: 0.0))
            self.addConstraint(NSLayoutConstraint(item: titleLable, attribute: .top, relatedBy: .equal, toItem: self, attribute: .top, multiplier: 1.0, constant: 15.0))
            self.addConstraint(NSLayoutConstraint(item: titleLable, attribute: .width, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1.0, constant: titleLableWidth))
            self.addConstraint(NSLayoutConstraint(item: titleLable, attribute: .height, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1.0, constant: titleLableHeight))
            
            return titleLable
    }()
    
    lazy var statusLabel : UILabel = {
        
        let statusLabel = UILabel()
            statusLabel.translatesAutoresizingMaskIntoConstraints = false
            self.addSubview(statusLabel)
            
            statusLabel.font = UIFont.systemFont(ofSize: 12, weight: UIFont.Weight.regular)
            statusLabel.textColor  = UIColor.init(red: 127/255.0, green: 127/255.0, blue: 127/255.0, alpha: 1/1.0)
            
            let statusLabelWidth : CGFloat = 121
            let statusLabelHeight : CGFloat = 17
            
            self.addConstraint(NSLayoutConstraint(item: statusLabel, attribute: .left, relatedBy: .equal, toItem: self, attribute: .left, multiplier: 1.0, constant: 16))
            self.addConstraint(NSLayoutConstraint(item: statusLabel, attribute: .right, relatedBy: .equal, toItem: self, attribute: .right, multiplier: 1.0, constant: 0.0))
            self.addConstraint(NSLayoutConstraint(item: statusLabel, attribute: .top, relatedBy: .equal, toItem: titleLable, attribute: .bottom, multiplier: 1.0, constant: 5.0))
            self.addConstraint(NSLayoutConstraint(item: statusLabel, attribute: .width, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1.0, constant: statusLabelWidth))
            self.addConstraint(NSLayoutConstraint(item: statusLabel, attribute: .height, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1.0, constant: statusLabelHeight))
            
            return statusLabel
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        translatesAutoresizingMaskIntoConstraints = false
        self.contentView.translatesAutoresizingMaskIntoConstraints = false
        
        
        
        self.backgroundColor = UIColor.white
        self.layer.cornerRadius = 10
        
        self.layer.masksToBounds = false
        self.layer.shadowColor = UIColor.black.cgColor
        self.layer.shadowOpacity = 0.1
        self.layer.shadowOffset = CGSize(width: 0, height: 4)
        self.layer.shadowRadius = 10
        updateCellSelection()
    }
    
    func updateCellSelection()  {
        if(selectedCell == true){
            selectedCellImageView.isHidden = false
        } else {
            selectedCellImageView.isHidden = true
        }
        
    }
    
    func configureCell( title : String ) {
        self.titleLable.text = title
        self.statusLabel.text = "available"
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}
