//
//  CanteenView.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 06/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import Foundation
import ZCCoreFramework

class GradientButton: UIButton {
    
    lazy var gradientLayer: CAGradientLayer = {
        let gradientLayer = CAGradientLayer()
        gradientLayer.colors = [UIColor(red: 255/255.0, green: 119/255.0, blue: 119/255.0, alpha: 1/1.0).cgColor,UIColor(red: 209/255.0, green: 178/255.0, blue: 135/255.0, alpha: 1/1.0).cgColor]
        gradientLayer.startPoint = CGPoint(x: 0, y: 0)
        gradientLayer.endPoint = CGPoint(x: 1, y: 0)
        gradientLayer.frame = CGRect(x: 0, y: 0, width: self.frame.size.width, height: 43)
        gradientLayer.cornerRadius = 20
        self.layer.insertSublayer(gradientLayer, at: 0)

        return gradientLayer
    }()
    
    override func layoutSubviews() {
        super.layoutSubviews()
        gradientLayer.bounds = self.bounds
    }
}

private let headerreuseIdentifier = "Header"
private var canteenCellWidth : CGFloat = 192
private let canteenCellHeight : CGFloat = 72
private let selectedFoodItemsHeaderHeight :CGFloat = 25
private let cellMinimumLineSpacing : CGFloat = 10

class CanteenView: UIView {
    
    weak var formViewControllerDelegate: FormViewController? = nil
    
    var unAlteredForm : Form?
    var form : Form?
    var fields: [FieldProtocol] = []
    var selectedCoices : [Choice] = []
    
    
    
    
    static var canteenCellReuseIds : String {
        return String(describing: self)
    }
    
    lazy var submitButton : UIButton = {
            let button = GradientButton()
            button.translatesAutoresizingMaskIntoConstraints = false
            button.addTarget(self, action: #selector(submitForm(_:)), for: .touchUpInside)
            button.layer.cornerRadius = 19
            button.titleLabel?.font = UIFont.boldSystemFont(ofSize: 14)
            button.setTitleColor(UIColor.white, for: .normal)
            button.setTitle("Order Now", for: UIControl.State.normal)
            button.layer.shadowColor = UIColor.init(red: 68.0/255.0, green: 60.0/255.0, blue: 60.0/255.0, alpha: 0.15).cgColor
            button.layer.shadowOffset = CGSize(width: 0.0, height: 4.0)
            button.layer.shadowOpacity = 1.0
            button.layer.masksToBounds = false

            addSubview(button)
            return button
    }()
    
    
    lazy var blackThemeview : UIView = {
            let blackThemeview = UIView()
            blackThemeview.backgroundColor = UIColor.init(red: 0/255.0, green: 0/255.0, blue: 0/255.0, alpha: 0.3)
            blackThemeview.translatesAutoresizingMaskIntoConstraints = false
            formViewControllerDelegate?.view.addSubview(blackThemeview)
            let tapGesture = UITapGestureRecognizer(target: self, action: #selector(self.dismissBlackThemeView))
            blackThemeview.addGestureRecognizer(tapGesture)
        
         NSLayoutConstraint.activate([blackThemeview.rightAnchor.constraint(equalTo: (formViewControllerDelegate?.view.rightAnchor)!),blackThemeview.leftAnchor.constraint(equalTo: (formViewControllerDelegate?.view.leftAnchor)!),blackThemeview.bottomAnchor.constraint(equalTo: (formViewControllerDelegate?.view.bottomAnchor)!),blackThemeview.topAnchor.constraint(equalTo: (formViewControllerDelegate?.view.topAnchor)!)])
        
            return blackThemeview
    }()
    
    
    lazy var selectedFoodsView : UIView = {
            let selectedFoodsView = UIView()
        selectedFoodsView.layer.cornerRadius = 15
        selectedFoodsView.layer.masksToBounds = true
        selectedFoodsView.clipsToBounds = true
            selectedFoodsView.translatesAutoresizingMaskIntoConstraints = false
            selectedFoodsView.backgroundColor = UIColor.init(red: 250/255.0, green: 250/255.0, blue: 250/255.0, alpha: 1/1.0)
            formViewControllerDelegate?.view.addSubview(selectedFoodsView)
            
            let selectedFoodsViewHeight : CGFloat =  getHeightForSelectedFoodView()
        
            NSLayoutConstraint.activate([selectedFoodsView.rightAnchor.constraint(equalTo: (formViewControllerDelegate?.view.rightAnchor)!),selectedFoodsView.leftAnchor.constraint(equalTo: (formViewControllerDelegate?.view.leftAnchor)!),selectedFoodsView.bottomAnchor.constraint(equalTo: (formViewControllerDelegate?.view.bottomAnchor)!),selectedFoodsView.heightAnchor.constraint(equalToConstant: selectedFoodsViewHeight)])
            
            
            let orderedView = OrderedFoodsView(form: form)
            orderedView.selectedCoices = self.selectedCoices
            orderedView.translatesAutoresizingMaskIntoConstraints = false
            selectedFoodsView.addSubview(orderedView)
            
            NSLayoutConstraint.activate([orderedView.rightAnchor.constraint(equalTo: selectedFoodsView.rightAnchor),orderedView.leftAnchor.constraint(equalTo: selectedFoodsView.leftAnchor),orderedView.bottomAnchor.constraint(equalTo: selectedFoodsView.bottomAnchor),orderedView.topAnchor.constraint(equalTo: selectedFoodsView.topAnchor)])
        
        return selectedFoodsView
    }()
    

   lazy var foodItemsCollectionView: UICollectionView =  {
        let layout: UICollectionViewFlowLayout = UICollectionViewFlowLayout()
        layout.sectionInset = UIEdgeInsets(top: 20, left: 10, bottom: 10, right: 10)
        layout.scrollDirection = .vertical
        layout.minimumInteritemSpacing = 5
    
        let width : CGFloat = self.frame.size.width
        let hight : CGFloat = 25
        layout.headerReferenceSize = CGSize(width: width, height: hight)
        
        let collectionview = UICollectionView.init(frame: .zero, collectionViewLayout: layout)
        collectionview.delaysContentTouches = false
        collectionview.translatesAutoresizingMaskIntoConstraints = false
        collectionview.backgroundColor = UIColor.init(red: 250/255.0, green: 250/255.0, blue: 250/255.0, alpha: 1/1.0)
        collectionview.delegate = self
        collectionview.dataSource = self
        collectionview.register(CanteenCell.self, forCellWithReuseIdentifier: CanteenView.canteenCellReuseIds)
        collectionview.register(FoodItemsCollectionHeaderView.self, forSupplementaryViewOfKind: UICollectionView.elementKindSectionHeader, withReuseIdentifier: headerreuseIdentifier)
        self.addSubview(collectionview)
    
        return collectionview
    }()
    
    init(form : Form?) {
        super.init(frame: .zero)
        
        backgroundColor = UIColor.init(red: 250/255.0, green: 250/255.0, blue: 250/255.0, alpha: 1/1.0)
        self.translatesAutoresizingMaskIntoConstraints = false
        
        if let form = form {
            self.form = form
            unAlteredForm = form
            filterFields()
            configureView()
        }
    }
    
    func filterFields() {
        self.fields = []
        let fields : [FieldProtocol] = (form?.fields)!
        for  field in fields {
            if(field.fieldType == FieldType.checkbox || field.fieldType == FieldType.multiselect) {
                self.fields.append(field)
            }
        }
    }
    
    func configureView() {
        
        // collectionView: constraints
        NSLayoutConstraint.activate([foodItemsCollectionView.rightAnchor.constraint(equalTo: self.rightAnchor),foodItemsCollectionView.leftAnchor.constraint(equalTo: self.leftAnchor),foodItemsCollectionView.bottomAnchor.constraint(equalTo: self.bottomAnchor),foodItemsCollectionView.topAnchor.constraint(equalTo: self.topAnchor)])
        
       
        
        // submitButton: constraints
        let submitButtonHeight: CGFloat = 43
        let submitButtonWidth: CGFloat = 134
        let submitButtonBottomPedding: CGFloat = -40
        
        NSLayoutConstraint.activate([
            submitButton.bottomAnchor.constraint(equalTo: self.bottomAnchor, constant: submitButtonBottomPedding),
            submitButton.centerXAnchor.constraint(equalTo: self.centerXAnchor),
            submitButton.heightAnchor.constraint(equalToConstant: submitButtonHeight),
            submitButton.widthAnchor.constraint(equalToConstant: submitButtonWidth)
            ])
        
        reloadFoodCollectionView()
        adjustCellWidth()
    }
    
    
    func adjustCellWidth() {
        let cellWidth : CGFloat = 192
        let widthRequiredByCell : CGFloat = cellWidth*2 + cellMinimumLineSpacing 
        let screenWidth = UIScreen.main.bounds.size.width
        if(widthRequiredByCell >= screenWidth){
            canteenCellWidth = cellWidth - (widthRequiredByCell - screenWidth)
        }
    }
    
    
    func reloadFoodCollectionView() {
        foodItemsCollectionView.reloadData()
    }
    
    
    func  refreshForm()  {
        form = unAlteredForm
        formViewControllerDelegate?.refreshFormUI(form: form!)
    }
    
    
    @objc func submitForm(_ sender: UIButton) {
        formViewControllerDelegate?.showActionLoading(show: true)
        updateSelectedChoices()
        if let form = form {
            ZohoCreatorAPIHandler.submit(form: form) { (result) in
                self.formViewControllerDelegate?.showActionLoading(show: false)
                switch result {
                case .failure(let error):
                    self.formViewControllerDelegate?.showMessage(message: error.localizedDescription)
                case .success(let value):do {
                    print (value)
                    self.getAllSelectedFoodItems()
                    self.showSelectedFoodItems()
                    }
                }
            }
        }
    }
    
    
    func getAllSelectedFoodItems() {
        selectedCoices = []
        for field in self.fields {
            let field = field as! CheckboxField
            let fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            let choices: [Choice] = fieldValue.selectedChoices
            selectedCoices.append(contentsOf: choices)
        }
    }
    
    // since field model in forms.fields is "pass by value" we need to update the canges made
    func updateSelectedChoices() {
        let originalFields : [FieldProtocol] = (form?.fields)!
        for  originalField in originalFields {
            for filteredField in self.fields {
                if(originalField.linkName == filteredField.linkName) {
                    if let index = form?.getIndex(linkName: filteredField.linkName) {
                        self.form?.setField(filteredField, at: index)
                    }
                }
            }
        }
    }
    
    func showSelectedFoodItems() {
        blackThemeview.alpha = 0
        selectedFoodsView.alpha = 0
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations:{
                self.blackThemeview.alpha = 1
                self.selectedFoodsView.alpha = 1
        })
    }
    
    func getHeightForSelectedFoodView( ) -> CGFloat{
       let foodViewCalculatedHeight = calculateHeightForSelectedFoodView()
       var minimumHeight : CGFloat = 302
        
       if(foodViewCalculatedHeight > minimumHeight) {
         minimumHeight = foodViewCalculatedHeight
        }
        return minimumHeight
    }
    
    func  calculateHeightForSelectedFoodView() -> CGFloat{
        let screenWidth = frame.size.width
        let numberOfCellsInRow = NSInteger(screenWidth / canteenCellWidth)
        let totalSelectedItems = selectedCoices.count
        var totalRows  =  NSInteger(totalSelectedItems / numberOfCellsInRow)
        let remainder = totalSelectedItems % numberOfCellsInRow
        let selectedFoodsViewHeaderSpacing : CGFloat = 40
        
        if(remainder > 0){
            totalRows = totalRows + 1
        }
        
        return (canteenCellHeight + cellMinimumLineSpacing) * CGFloat(totalRows) + selectedFoodItemsHeaderHeight + selectedFoodsViewHeaderSpacing
    }
    
    @objc func dismissBlackThemeView() {
        blackThemeview.removeFromSuperview()
        selectedFoodsView.removeFromSuperview()
        refreshForm()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

extension CanteenView : UICollectionViewDelegate {
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        let canteenCell =  collectionView.cellForItem(at: indexPath) as! CanteenCell
        canteenCell.isSelectedCell = true
        if(indexPath.section == 0){
            
            var field = fields[0] as! CheckboxField
            var fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            if(canteenCell.selectedCell == true) {
                fieldValue.selectedChoices.append(canteenCell.choice!)
            } else {
                if let index = fieldValue.selectedChoices.index(of: canteenCell.choice!) {
                    fieldValue.selectedChoices.remove(at: index)
                }
            }
            // since fieldValue model is "pass by value" we need to update the canges made
            field.fieldValue = fieldValue
            fields.remove(at: 0)
            fields.insert(field, at: 0)
        }else {
            var field = fields[1] as! CheckboxField
            var fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            
            if(canteenCell.selectedCell == true){
                fieldValue.selectedChoices.append(canteenCell.choice!)
            } else {
                if let index = fieldValue.selectedChoices.index(of: canteenCell.choice!) {
                    fieldValue.selectedChoices.remove(at: index)
                }
            }
            // since fieldValue model is "pass by value" we need to update the canges made
            field.fieldValue = fieldValue
            fields.remove(at: 1)
            fields.insert(field, at: 1)
        }
    }
}

extension CanteenView : UICollectionViewDataSource {
    
    
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return fields.count
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if (section == 0) {
            let field = fields[0] as! CheckboxField
            let fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            let choices: [Choice] = fieldValue.choices
            return choices.count
        }
        let field = fields[1] as! CheckboxField
        let fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
        let choices: [Choice] = fieldValue.choices
        return choices.count
    }
    
    func collectionView(_ collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, at indexPath: IndexPath) -> UICollectionReusableView {
        let headerView : FoodItemsCollectionHeaderView = collectionView.dequeueReusableSupplementaryView(ofKind: kind, withReuseIdentifier: headerreuseIdentifier, for: indexPath) as! FoodItemsCollectionHeaderView
        
        if(indexPath.section == 0) {
            headerView.configureHeader(title: "TODAY’S SPECIAL")
        } else {
            headerView.configureHeader(title: "Casseroles")
        }
        return headerView
    }

    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: CanteenView.canteenCellReuseIds, for: indexPath) as! CanteenCell
        if(indexPath.section == 0) {
            let field = fields[0] as! CheckboxField
            let fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            let choices: [Choice] = fieldValue.choices
            let choice = choices[indexPath.row]
            cell.choice = choice
            cell.configureCell(title: choice.key)
        } else if (indexPath.section == 1){
            let field = fields[1] as! CheckboxField
            let fieldValue:MultiSelectChoiceFieldValue  = field.fieldValue
            let choices: [Choice] = fieldValue.choices
            let choice = choices[indexPath.row]
            cell.choice = choice
            cell.configureCell(title: choice.key)
        }
        return cell
    }
}

extension CanteenView : UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        return CGSize(width: canteenCellWidth ,height: canteenCellHeight)
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return cellMinimumLineSpacing
    }

    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, referenceSizeForFooterInSection section: Int) -> CGSize {
        return CGSize(width: 0,height: 0)
    }
}
