//
//  OrderedFoodsView.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 07/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//



import Foundation
import ZCCoreFramework

private var orderedCellWidth : CGFloat = 192
private let headerreuseIdentifier = "Header"
private let cellMinimumLineSpacing : CGFloat = 10

class OrderedFoodsView: UIView {
    
    weak var formViewControllerDelegate: FormViewController? = nil
    
    var form : Form?
    var fields: [FieldProtocol] = Array()
    var selectedCoices : [Choice] = Array()
    
    lazy var orderedFoodCollection : UICollectionView = {
            let layout: UICollectionViewFlowLayout = UICollectionViewFlowLayout()
            layout.sectionInset = UIEdgeInsets(top: 20, left: 10, bottom: 10, right: 10)
            layout.scrollDirection = .vertical
            layout.minimumInteritemSpacing = 5
            
            let width : CGFloat = self.frame.size.width
            let hight : CGFloat = 25
            layout.headerReferenceSize = CGSize(width: width, height: hight)
            
            
            
            let collectionview = UICollectionView.init(frame: .zero, collectionViewLayout: layout)
            collectionview.delaysContentTouches = false
            collectionview.translatesAutoresizingMaskIntoConstraints = false
            collectionview.backgroundColor = UIColor.init(red: 250/255.0, green: 250/255.0, blue: 250/255.0, alpha: 1/1.0)
            collectionview.delegate = self
            collectionview.dataSource = self
            collectionview.register(CanteenCell.self, forCellWithReuseIdentifier: "canteencell")
            collectionview.register(OrderedFoodsCollectionHeaderView.self, forSupplementaryViewOfKind: UICollectionView.elementKindSectionHeader, withReuseIdentifier: headerreuseIdentifier)
            self.addSubview(collectionview)
            
        NSLayoutConstraint.activate([collectionview.rightAnchor.constraint(equalTo: self.rightAnchor),collectionview.leftAnchor.constraint(equalTo: self.leftAnchor),collectionview.bottomAnchor.constraint(equalTo: self.bottomAnchor),collectionview.topAnchor.constraint(equalTo: self.topAnchor,constant: 10)])
            return collectionview
    }()
    
    init(form : Form?) {
        super.init(frame: .zero)
        
        if let form = form {
            self.form = form
            configureView()
        }
        
    }
    
    func configureView() {
        
        backgroundColor = UIColor.init(red: 250/255.0, green: 250/255.0, blue: 250/255.0, alpha: 1/1.0)
        self.translatesAutoresizingMaskIntoConstraints = false
        orderedFoodCollection.reloadData()
        adjustCellWidth()
    }
    
    func adjustCellWidth() {
        let cellWidth : CGFloat = 192
        let widthRequiredByCell : CGFloat = cellWidth*2 + cellMinimumLineSpacing
        let screenWidth = UIScreen.main.bounds.size.width
        if(widthRequiredByCell >= screenWidth){
            orderedCellWidth = cellWidth - (widthRequiredByCell - screenWidth)
        }
    }
    
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

extension OrderedFoodsView : UICollectionViewDataSource {
    
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return selectedCoices.count
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        let widthValue : CGFloat = orderedCellWidth
        let heightValue : CGFloat = 72
        return CGSize(width: widthValue,height: heightValue)
    }
    
    func collectionView(_ collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, at indexPath: IndexPath) -> UICollectionReusableView {
        let headerView : OrderedFoodsCollectionHeaderView = collectionView.dequeueReusableSupplementaryView(ofKind: kind, withReuseIdentifier: headerreuseIdentifier, for: indexPath) as! OrderedFoodsCollectionHeaderView
        if(indexPath.section == 0) {
            headerView.configureHeader(title: "My Orders")
        }
        return headerView
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "canteencell", for: indexPath) as! CanteenCell
        if(indexPath.section == 0) {
            
            let choice = selectedCoices[indexPath.row]
            cell.choice = choice
            cell.configureCell(title: choice.key)
        }
        return cell
    }
}

extension OrderedFoodsView : UICollectionViewDelegateFlowLayout{
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return cellMinimumLineSpacing
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, referenceSizeForFooterInSection section: Int) -> CGSize {
        return CGSize(width: 0,height: 0)
    }
}
