//
//  ViewController.swift
//  Complaints
//
//  Created by ZOHO corporation  on 07/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework
import MobileCoreServices

protocol SingleSelectedChoice{
    func addchoiceinSingleSelectField(selectedChoice: Choice)
}

class ComplaintsViewController: UIView {
    
    var tableView = UITableView()
    weak var formViewControllerDelegate: FormViewController? = nil
    
    lazy var paranteView : UIView = {
        var paranteView = UIView()
        addSubview(paranteView)
        let safeAreaView = self
        paranteView.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint(item: paranteView, attribute: .leading, relatedBy: .equal, toItem: safeAreaView, attribute: .leading, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: paranteView, attribute: .top, relatedBy: .equal, toItem: safeAreaView, attribute: .top, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: paranteView, attribute: .trailing, relatedBy: .equal, toItem: safeAreaView, attribute: .trailing, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: paranteView, attribute: .bottom, relatedBy: .equal, toItem: safeAreaView, attribute: .bottom, multiplier: 1, constant: 0).isActive = true
        return paranteView
    }()
    
    
    
    var form : Form?
    var fields: [FieldProtocol] = Array()
    var selectedChoices : [Choice] = Array()
    var selectedCell:Int?
    var filename:String? = ""
    var selectedChoice: Choice?
    
    
    init(form : Form?)
    {
        super.init(frame: .zero)
        
        if let form = form {
            self.form = form
            filterFields()
            configureView()
        }
        
    }
    
    
    func filterFields(){
        let fields : [FieldProtocol] = (form?.fields)!
        for  field in fields {
            if(field.fieldType == FieldType.dropdown || field.fieldType == FieldType.multiLine || field.fieldType == FieldType.decisionBox) || field.fieldType == .fileUpload {
                self.fields.append(field)            
            }
        }
        dump(self.fields)
    }
    
    
    func  configureView(){
        
        
        tableView.delegate = self
        tableView.dataSource = self
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.keyboardDismissMode = .interactive
        tableView.register(ServiceCell.self, forCellReuseIdentifier: "ServiceCell")
        tableView.register(ComplaintEntryCell.self, forCellReuseIdentifier: "complaintEntryCell")
        tableView.register(ReportCell.self, forCellReuseIdentifier: "ReportCell")
        tableView.register(SubmitButtoncell.self, forCellReuseIdentifier: "SubmitButtoncell")
        self.paranteView.addSubview(tableView)
        tableView.separatorStyle = .none
        tableView.backgroundColor = UIColor(red: 239/255, green: 239/255, blue: 244/255, alpha: 1)
        NSLayoutConstraint(item: tableView, attribute: .leading, relatedBy: .equal, toItem: paranteView, attribute: .leading, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: tableView, attribute: .top, relatedBy: .equal, toItem: paranteView, attribute: .top, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: tableView, attribute: .trailing, relatedBy: .equal, toItem: paranteView, attribute: .trailing, multiplier: 1, constant: 0).isActive = true
        NSLayoutConstraint(item: tableView, attribute: .bottom, relatedBy: .equal, toItem: paranteView, attribute: .bottom, multiplier: 1, constant: 0).isActive = true
        let footer = UIView()
        tableView.tableFooterView = footer
        
        tableView.rowHeight = UITableView.automaticDimension
        tableView.estimatedRowHeight = 54
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func multiLine( text: String){
        
        if let field = fields[1].setValueFor(value: InputValues.textFieldValue(fieldValue: text)){
            
            fields[1] = field
        }
    }
    
    @objc func submitForm() {
        if var form = form {
            for field in fields {
                if let index = form.getIndex(linkName: field.linkName){
                    form.setField(field, at: index)
                }
            }
            formViewControllerDelegate?.showActionLoading(show: true)
            ZCFormAPIService.submit(form: form) { (response) in
                self.formViewControllerDelegate?.showActionLoading(show: false)
                self.formViewControllerDelegate?.showSuccessComplainRaiseView()
                switch response {
                case .success(let successResp):
                    dump(successResp)
                case .failure(_):
                    break
                }
            }
        }
    }
    
    
    @objc func avilableChoiceInService(){
        
        let lister =  ComplainServicesLister()
        lister.serviceSelector = self
        lister.choiceList = selectedChoices
        lister.barTitle = "Select Choice "
        lister.selectedChoice = selectedChoice
        formViewControllerDelegate?.navigationController?.pushViewController(lister, animated: true)
        
    }
    
    
    func switchValue(_ isDisabled:Bool,_ tag : Int){
        
        if let field = fields[tag].setValueFor(value: InputValues.decisionFieldValue(fieldValue: isDisabled)){
            
            fields[tag] = field
        }
    }
    
}

extension ComplaintsViewController: UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.cellForRow(at: indexPath)?.selectionStyle = .none
    }
}


extension ComplaintsViewController: UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        if section == 1{
            let footerview = UIView.init(frame: CGRect.init(x: 0, y: 0, width: self.paranteView.frame.size.width, height: 30))
            footerview.autoresizingMask = [.flexibleWidth]
            return footerview
        }
        return nil
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == 1{
            
            return 30
        }
        return CGFloat.leastNonzeroMagnitude
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == 0{
            return 5
        }else{
            return 1
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.section == 0{
            if indexPath.row == 0 {
                let cell = tableView.dequeueReusableCell(withIdentifier: "ServiceCell") as! ServiceCell
                cell.delegate = self
                NSLayoutConstraint(item: cell.rightarrow, attribute: .top, relatedBy: .equal, toItem: cell.rightView, attribute: .top, multiplier: 1, constant: 15).isActive = true
                NSLayoutConstraint(item: cell.rightarrow, attribute: .bottom, relatedBy: .equal, toItem: cell.rightView, attribute: .bottom, multiplier: 1, constant: -15).isActive = true
                if case InputValues.singleSelectChoiceFieldValue(let fieldValue) = fields[indexPath.row].getValue(){
                    selectedChoices = fieldValue.choices
                    if let selectedChoice = fieldValue.selectedChoice{
                        cell.selectedLabel.text = selectedChoice.value
                    }
                    else{
                        cell.selectedLabel.text = nil
                    }
                }
                let tapGesture = UITapGestureRecognizer(target: self, action: #selector(self.avilableChoiceInService))
                cell.rightView.addGestureRecognizer(tapGesture)
                selectedCell = indexPath.row
                cell.nameLabel.text = fields[indexPath.row].displayName
                return cell
            }else if indexPath.row == 1{
                let cell = tableView.dequeueReusableCell(withIdentifier: "complaintEntryCell") as! ComplaintEntryCell
                cell.nameLabel.text = fields[indexPath.row].displayName
                
                if case InputValues.textFieldValue(let fieldValue) = fields[indexPath.row].getValue() {
                    
                    cell.entryTextView.text = fieldValue
                }
                
                cell.delegate = self
                return cell
            }else if indexPath.row == 2{
                let cell = tableView.dequeueReusableCell(withIdentifier: "ServiceCell") as! ServiceCell
                NSLayoutConstraint(item: cell.rightarrow, attribute: .top, relatedBy: .equal, toItem: cell.rightView, attribute: .top, multiplier: 1, constant: 24).isActive = true
                NSLayoutConstraint(item: cell.rightarrow, attribute: .bottom, relatedBy: .equal, toItem: cell.rightView, attribute: .bottom, multiplier: 1, constant: -24).isActive = true
                let tapGesture = UITapGestureRecognizer(target: self, action: #selector(self.addDocumentPicker))
                cell.rightView.addGestureRecognizer(tapGesture)
                cell.nameLabel.text = fields[indexPath.row].displayName
                cell.delegate = self
                cell.selectedLabel.text = filename
                return cell
            }else if indexPath.row == 3{
                let cell = tableView.dequeueReusableCell(withIdentifier: "ReportCell") as! ReportCell
                cell.nameLabel.text = fields[indexPath.row].displayName
                
                cell.switchView.tag = indexPath.row
                cell.delegate = self
                if case InputValues.decisionFieldValue(let fieldValue) = fields[indexPath.row].getValue() {
                    
                    cell.switchView.setOn(fieldValue, animated: true)
                }
                return cell
            }else {
                let cell = tableView.dequeueReusableCell(withIdentifier: "ReportCell") as! ReportCell
                cell.nameLabel.text = fields[indexPath.row].displayName
                
                cell.switchView.tag = indexPath.row
                cell.delegate = self
                if case InputValues.decisionFieldValue(let fieldValue) = fields[indexPath.row].getValue() {
                    
                    cell.switchView.setOn(fieldValue, animated: true)
                }
                return cell
            }
            
        }else {
            
            let cell = tableView.dequeueReusableCell(withIdentifier: "SubmitButtoncell") as! SubmitButtoncell
            cell.submit.addTarget(self, action: #selector(submitForm), for: .touchUpInside)
            return cell
        }
    }
}


extension ComplaintsViewController : SingleSelectedChoice {
    
    func addchoiceinSingleSelectField(selectedChoice: Choice) {
        
        if(self.selectedChoice?.key == selectedChoice.key){
            self.selectedChoice = nil
        }else {
            self.selectedChoice = selectedChoice
        }
        choiceValueSetter()
    }
    
    func choiceValueSetter() {
        
        if let index = selectedCell,
            case InputValues.singleSelectChoiceFieldValue(let fieldValue) = fields[index].getValue(){
            
            let inputValue = InputValues.singleSelectChoiceFieldValue(fieldValue: SingleSelectChoiceFieldValue(choices: fieldValue.choices, selectedChoice: self.selectedChoice))
            fields[index] = fields[index].setValueFor(value: inputValue) ?? fields[index]
            tableView.reloadData()
        }
    }
    
}


extension ComplaintsViewController : UIDocumentPickerDelegate {
    
    @objc func addDocumentPicker(){
        let documentPicker = UIDocumentPickerViewController(documentTypes: [kUTTypeItem as String], in: .import)
        documentPicker.delegate = self
        documentPicker.allowsMultipleSelection = false
        formViewControllerDelegate?.present(documentPicker, animated: true, completion: nil)
    }
    
    @available(iOS 11,*)
    func documentPicker(_ controller: UIDocumentPickerViewController, didPickDocumentsAt urls: [URL]){
        print(urls[0].lastPathComponent)
        
        let filedata = try! Data(contentsOf: urls[0])
        ZCFormAPIService.upload(media: filedata, withFileName: urls[0].lastPathComponent, for: .field(linkName: "Attachments_Any"), form: form!) { (result) in
            switch result {
            case .success(let response):
                print(response)
                
                DispatchQueue.main.async {
                    self.filename = response.fileName
                    
                    if let field = self.fields[2].setValueFor(value: InputValues.mediaFieldValue(fieldValue: MediaFieldValue(fileData: nil, sourcePath: self.filename))){
                        
                        self.fields[2] = field
                    }
                    
                    self.tableView.reloadData()
                }
                
            case .failure(let error):
                
                print(error)
                
            }
        }
        
    }
    
    
    func documentPicker(_ controller: UIDocumentPickerViewController, didPickDocumentAt url: URL) {
        let filedata = try! Data(contentsOf: url)
        ZCFormAPIService.upload(media: filedata, withFileName: url.lastPathComponent, for: .field(linkName: "Attachments_Any"), form: form!) { (result) in
            switch result {
            case .success(let response):
                print(response)
                
                DispatchQueue.main.async {
                    self.filename = response.fileName
                    if let field = self.fields[2].setValueFor(value: InputValues.mediaFieldValue(fieldValue: MediaFieldValue(fileData: nil, sourcePath: self.filename))){
                        
                        self.fields[2] = field
                    }
                    self.tableView.reloadData()
                }
                
            case .failure(let error):
                
                print(error)
                
            }
        }
    }
}
