//
//  FormViewController.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 06/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

enum FormType :String{
    case Supplies
    case Shuttles
    case Canteens
    case Complaints
}

class FormViewController: UIViewController {
    var currentSection : Section
    var profileHolderView = ProfileInfoDisplayView()
    let supplyView = SuppliesView()
    var blurView:UIView?
    var currentFormType : FormType?
    var currentForm : Form?
    var canteenView  : CanteenView?
    var shuttleView  : ShuttleView?
    
    var formsHolderView = UIView()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.navigationController?.setNavigationBarHidden(true, animated: false)
        
        self.view.backgroundColor = UIColor.white
        
        self.congigureFieldInfoView()
        self.loadForm()
    }
    init(withSection section: Section)
    {
        self.currentSection = section
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    func loadForm() {
        var currentFormComponent : Component?
        for formComponent in self.currentSection.components {
            if formComponent.type == .form{
                currentFormComponent = formComponent
                break
            }
        }
        
        if let component = currentFormComponent{
        
            let formInfo = FormInfo(linkName: component.linkName, displayName: component.displayName, appLinkName: component.appLinkName, appOwner: component.appOwner, openUrlInfo: component.openUrlInfo)
        
            self.showPageLoader(show: true)
            ZohoCreatorAPIHandler.fetch(for: formInfo) { (result) in
                self.showPageLoader(show: false)
                switch result {
                case .failure( _): break
                    
                case .success(let value):do {
                    print (value.alteredForm)
                    let formLinkName = value.alteredForm.linkName
                    self.currentForm = value.alteredForm
                    
                    switch formLinkName{
                    case "Supplies":
                        self.currentFormType = .Supplies
                        self.configureViewForSupplies(forForm: value.alteredForm)
                    case "Shuttle":
                        self.currentFormType = .Shuttles
                        self.configureViewForShuttles(form: value.alteredForm)
                    case "Canteen":
                        self.configureViewForCanteens(form:value.alteredForm)
                    case "Complaints":
                        self.currentFormType = .Complaints
                        self.configureViewForComplaints(forForm: value.alteredForm)
                    default: break
                    }
                    }
                }
            }
        }
    }
    
    func congigureFieldInfoView(){
        profileHolderView.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(profileHolderView)
        
        profileHolderView.leftAnchor.constraint(equalTo: self.view.leftAnchor, constant: 0).isActive = true
        profileHolderView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: 0).isActive = true
        profileHolderView.topAnchor.constraint(equalTo: self.view.topAnchor, constant: 0).isActive = true
        profileHolderView.heightAnchor.constraint(equalToConstant: CGFloat(SectionListingViewController.profileHolderViewHeight)).isActive = true
        
        formsHolderView.layer.cornerRadius = 15
        formsHolderView.layer.masksToBounds = true
        formsHolderView.backgroundColor = UIColor.white
        formsHolderView.clipsToBounds = true
        
        formsHolderView.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(formsHolderView)
        
        formsHolderView.leftAnchor.constraint(equalTo: self.view.leftAnchor, constant: 0).isActive = true
        formsHolderView.topAnchor.constraint(equalTo: profileHolderView.bottomAnchor, constant: -15).isActive = true
        formsHolderView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: 0).isActive = true
        formsHolderView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        
    }
    
    func handleBackButtonAction() {
        if let controllers = self.navigationController?.viewControllers, controllers.count > 1 {
            self.navigationController?.popViewController(animated: true)
        } else {
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    func handleNextButtonAction() {
        
        supplyView.handleNextButtonAction()
    }
    
    func cancelClicked() {
        if let blurView = blurView{
            if supplyView.isSupplyViewFormSubmitted{
                self.configureViewForSupplies(forForm: self.currentForm!)
            }
            blurView.removeFromSuperview()
        }
    }
    
    func resetSupplyForm() {
        if let blurView = blurView{
            if supplyView.isSupplyViewFormSubmitted{
                self.configureViewForSupplies(forForm: self.currentForm!)
            }
            blurView.removeFromSuperview()
        }
        self.configureViewForSupplies(forForm: self.currentForm!)
    }
    
    func addBlurView() -> UIView {
        blurView = UIView.init(frame: CGRect.init(x: 0, y: 0, width: self.view.frame.width, height: self.view.frame.height))
        blurView?.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        self.view.addSubview(blurView!)
        
        if !UIAccessibility.isReduceTransparencyEnabled {
            blurView!.backgroundColor = .clear
            
            let blurEffect = UIBlurEffect(style: .dark)
            let blurEffectView = UIVisualEffectView(effect: blurEffect)
            blurEffectView.frame = self.view.bounds
            blurEffectView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
            blurView!.addSubview(blurEffectView)
        } else {
            blurView!.backgroundColor = .black
        }
        return blurView!
    }
    
    func configureViewForSupplies(forForm form: Form){
        
        let supplyFieldInfo = SuppliesFormHelper.getSupplyFieldInfo()
        
        self.profileHolderView.showFieldInfo(withFieldName: supplyFieldInfo.title, andFieldDescription: supplyFieldInfo.description, buttonText: supplyFieldInfo.buttonTitle, withFormVC: self)
        supplyView.formVC = self
        supplyView.translatesAutoresizingMaskIntoConstraints = false
        formsHolderView.addSubview(supplyView)
        
        supplyView.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        supplyView.bottomAnchor.constraint(equalTo: formsHolderView.bottomAnchor, constant: 0).isActive = true
        supplyView.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        supplyView.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        
        supplyView.configureSuppliesView(withForm: form)
    }
    
    func configureViewForShuttles(form:Form){
        
        self.profileHolderView.showFieldInfo(withFieldName: "Shuttle", andFieldDescription: "Select Drop location and timing to continue", buttonText: nil, withFormVC: self)
        
        shuttleView = ShuttleView(form: form)
        shuttleView?.translatesAutoresizingMaskIntoConstraints = false
        formsHolderView.addSubview(shuttleView!)
        shuttleView?.formViewControllerDelegate = self
        shuttleView?.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        shuttleView?.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        shuttleView?.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        shuttleView?.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        
        shuttleView?.alpha = 0
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations:
            {
                self.shuttleView?.alpha = 1
        })
        { (finished) in
            
        }
    }
    
    func  showSuccessComplainRaiseView() {
        self.profileHolderView.showFieldInfo(withFieldName: "Complaints", andFieldDescription: "Our service superviser will contact you shortly", buttonText: nil, withFormVC: self)
        
        shuttleView?.removeFromSuperview()
        let successFulCabBBookingView = ComplainSuccessfullyRaisedView()
        successFulCabBBookingView.backgroundColor = UIColor.white
        formsHolderView.addSubview(successFulCabBBookingView)
        successFulCabBBookingView.translatesAutoresizingMaskIntoConstraints = false
        successFulCabBBookingView.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        successFulCabBBookingView.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        successFulCabBBookingView.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        successFulCabBBookingView.bottomAnchor.constraint(equalTo: formsHolderView.bottomAnchor, constant: 0).isActive = true
        
        successFulCabBBookingView.alpha = 0
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations:
            {
                successFulCabBBookingView.alpha = 1
        })
        { (finished) in
            
        }
    }
    
    func showSuccessCabBookingView(selectedTime: String) {
        
        self.profileHolderView.showFieldInfo(withFieldName: "Shuttle", andFieldDescription: "Select Drop location and timing to continue", buttonText: nil, withFormVC: self)
        
        shuttleView?.removeFromSuperview()
        let successFulCabBBookingView = SuccessfulBookingShuttleView()
        successFulCabBBookingView.backgroundColor = UIColor.white
        formsHolderView.addSubview(successFulCabBBookingView)
        successFulCabBBookingView.translatesAutoresizingMaskIntoConstraints = false
        successFulCabBBookingView.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        successFulCabBBookingView.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        successFulCabBBookingView.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        successFulCabBBookingView.bottomAnchor.constraint(equalTo: formsHolderView.bottomAnchor, constant: 0).isActive = true
        
        successFulCabBBookingView.alpha = 0
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations:
            {
                successFulCabBBookingView.alpha = 1
        })
        { (finished) in
            
        }
    }
    
    func refreshFormUI(form: Form) {
        canteenView!.removeFromSuperview()
        configureViewForCanteens(form: form)
    }
    
    func configureViewForCanteens(form:Form){
        self.profileHolderView.showFieldInfo(withFieldName: "Breakfast", andFieldDescription: "07:30 am - 10:45 am", buttonText: nil, withFormVC: self)
        
        canteenView = CanteenView(form: form)
        canteenView?.formViewControllerDelegate = self
        canteenView?.translatesAutoresizingMaskIntoConstraints = false
        formsHolderView.addSubview(canteenView!)
        
        canteenView?.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        canteenView?.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        canteenView?.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        canteenView?.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        
        canteenView?.alpha = 0
        self.showPageLoader(show: true)
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations:
            {
                self.showPageLoader(show: false)
                
        })
        { (finished) in
            self.canteenView?.alpha = 1
        }
        
    }
    func formSubmit(form:Form) {
        
    }
    
    func configureViewForComplaints(forForm form: Form) {
        
        self.profileHolderView.showFieldInfo(withFieldName: "Complaints", andFieldDescription: "Raise a complaint on particular services", buttonText:nil, withFormVC: self)
        
        let complaintView = ComplaintsViewController(form : form)
        complaintView.formViewControllerDelegate = self
        formsHolderView.addSubview(complaintView)
        complaintView.translatesAutoresizingMaskIntoConstraints = false
        complaintView.leftAnchor.constraint(equalTo: formsHolderView.leftAnchor, constant: 0).isActive = true
        complaintView.rightAnchor.constraint(equalTo: formsHolderView.rightAnchor, constant: 0).isActive = true
        complaintView.topAnchor.constraint(equalTo: formsHolderView.topAnchor, constant: 0).isActive = true
        complaintView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
}
