//
//  SectionListingViewController.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 05/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework
import ZCUIFramework

class SectionListingViewController: UIViewController, UICollectionViewDelegateFlowLayout,UICollectionViewDataSource {
    var currentApplication : Application
    var sectionListArray : SectionList?
    var isSectionDataReceived = false
    

    static var sectionListReuseId = "SectionListCollectionViewCell"
    static let profileHolderViewHeight = 200.0
    
    lazy var collectionView : UICollectionView = {
        let layout: UICollectionViewFlowLayout = UICollectionViewFlowLayout()
        layout.sectionInset = UIEdgeInsets(top: 10, left: 10, bottom: 10, right: 10)
        layout.minimumInteritemSpacing = 10
        layout.minimumLineSpacing = 10
        
        if #available(iOS 11.0, *) {
            layout.sectionInsetReference = .fromSafeArea
        }
        return UICollectionView(frame: CGRect.zero, collectionViewLayout: layout)
    }()
    var profileHolderView = ProfileInfoDisplayView()
    
    
    init(withApp application: Application)
    {
        self.currentApplication = application
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()

        self.view.backgroundColor = UIColor.init(red: 252.0/255.0, green: 252.0/255.0, blue: 252.0/255.0, alpha: 1)
        self.navigationController?.setNavigationBarHidden(true, animated: false)
        
        profileHolderView.translatesAutoresizingMaskIntoConstraints = false
        profileHolderView.configureView(shouldShowProfile: true)
        self.view.addSubview(profileHolderView)
        
        profileHolderView.leftAnchor.constraint(equalTo: self.view.leftAnchor, constant: 0).isActive = true
        profileHolderView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: 0).isActive = true
        profileHolderView.topAnchor.constraint(equalTo: self.view.topAnchor, constant: 0).isActive = true
        profileHolderView.heightAnchor.constraint(equalToConstant: CGFloat(SectionListingViewController.profileHolderViewHeight)).isActive = true
        
        let collectionViewHolderView = UIView()
        collectionViewHolderView.backgroundColor = UIColor.white
        collectionViewHolderView.layer.cornerRadius = 15
        collectionViewHolderView.layer.masksToBounds = true
        collectionViewHolderView.clipsToBounds = true
        
        collectionViewHolderView.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(collectionViewHolderView)
        
        collectionViewHolderView.leftAnchor.constraint(equalTo: self.view.leftAnchor, constant: 0).isActive = true
        collectionViewHolderView.topAnchor.constraint(equalTo: profileHolderView.bottomAnchor, constant: -15).isActive = true
        collectionViewHolderView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: 0).isActive = true
        collectionViewHolderView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        
        
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.backgroundColor = UIColor.init(red: 252.0/255.0, green: 252.0/255.0, blue: 252.0/255.0, alpha: 1)
        collectionView.translatesAutoresizingMaskIntoConstraints = false
        collectionViewHolderView.addSubview(collectionView)
        
        collectionView.register(CollectionViewCell.self, forCellWithReuseIdentifier: SectionListingViewController.sectionListReuseId)
        
        collectionView.leftAnchor.constraint(equalTo: collectionViewHolderView.leftAnchor, constant: 0).isActive = true
        collectionView.topAnchor.constraint(equalTo: collectionViewHolderView.topAnchor, constant: 0).isActive = true
        collectionView.rightAnchor.constraint(equalTo: collectionViewHolderView.rightAnchor, constant: 0).isActive = true
        collectionView.bottomAnchor.constraint(equalTo: collectionViewHolderView.bottomAnchor, constant: 0).isActive = true
        
        self.showPageLoader(show: true)
        ZohoCreatorAPIHandler.fetchMySectionList(for: currentApplication, completionHandler: { (result) in
            self.showPageLoader(show: false)
            switch result
            {
            case .failure(let error):
                self.showMessage(message: error.localizedDescription)
            case .success(let sectionList) :
                self.sectionListArray = sectionList
                self.isSectionDataReceived = true
                self.collectionView.reloadData()
                
            }
        }, shouldCacheResponse: true)
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if isSectionDataReceived{
            return sectionListArray!.sections.count
        }
        return 0
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView
            .dequeueReusableCell(withReuseIdentifier: SectionListingViewController.sectionListReuseId, for: indexPath) as! CollectionViewCell
        if let currentSection =  sectionListArray?.sections[indexPath.row]{
            cell.configureCell(forSection: currentSection)
        }
//        cell.textLabel.text = sectionListArray?.sections[indexPath.row].sectionName
        
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        return CGSize(width: (collectionView.bounds.width - 30) / 2, height: 200)
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        let selectedSection = self.sectionListArray!.sections[indexPath.row]
        if let formComponent = selectedSection.components.first(where: { $0.type == .form }) {
            switch formComponent.linkName {
            case "Shuttle", "Complaints":
                let controller = ZCUIService.getViewController(forForm: ComponentDetail.init(appOwner: formComponent.appOwner, appName: formComponent.appLinkName, componentLinkName: formComponent.componentLinkName))
                self.navigationController?.present(UINavigationController.init(rootViewController: controller), animated: true, completion: nil)
            default:
                let responseViewController = FormViewController.init(withSection: selectedSection)
                self.navigationController?.present(UINavigationController.init(rootViewController: responseViewController), animated: true, completion: nil)
            }
        }

    }
    
}

extension UIViewController {
    var ActivityIndicatorTag: Int { return 15439 }
    
    func showPageLoader(show:Bool){
        let activityIndicator = self.view.viewWithTag(ActivityIndicatorTag)
        if show {
            if let activityIndicator  = activityIndicator{
                (activityIndicator as! UIActivityIndicatorView).startAnimating()
            }
            else {
                let loader = UIActivityIndicatorView.init(frame: CGRect.init(x: 0, y: 0, width: self.view.frame.size.width, height: self.view.frame.size.height))
                loader.style = .gray
                loader.center = self.view.center
                
                let topPadding = floor(SectionListingViewController.profileHolderViewHeight / 2.0) - 5
                loader.center = CGPoint.init(x: self.view.center.x, y: self.view.center.y + CGFloat(topPadding))
                loader.tag = ActivityIndicatorTag
                self.view.addSubview(loader)
                
                loader.startAnimating()
            }
            
        } else {
            if let activityIndicator  = activityIndicator{
                let loc = activityIndicator as! UIActivityIndicatorView
                loc.stopAnimating()
            }
        }
    }
}
