//
//  ProfileInfoDisplayView.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 05/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class ProfileInfoDisplayView: UIView {
    lazy var profilebutton = UIButton()
    lazy var backButton = UIButton.init(type: .custom)
    var formVC : FormViewController?
    let fieldInfoLabel = UILabel()
    let fieldDescriptionInfoLabel = UILabel()
    let nextButton = UIButton()
    
    func configureView(shouldShowProfile : Bool)  {
        if shouldShowProfile {
            self.showProfileInfo()
        }
        else {
            
        }
    }
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.setBGColor()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    func setBGColor() {
        self.backgroundColor = UIColor.init(red: 73.0/255.0, green: 90.0/255.0, blue: 120.0/255.0, alpha: 1)
    }
    func showProfileInfo(){
        let leftPaddingProfileImage = 20
        let topPaddingProfileImage = 50
        let bottomPaddingProfileImage = 30
        if let image = UIImage(named: "user_profile"){
            profilebutton.setBackgroundImage(image, for: .normal)
        }
        
        profilebutton.translatesAutoresizingMaskIntoConstraints = false
        profilebutton.addTarget(self, action: #selector(logoutAction), for: .touchUpInside)
        self.addSubview(profilebutton)
        
        profilebutton.leftAnchor.constraint(equalTo: self.leftAnchor, constant: CGFloat(leftPaddingProfileImage)).isActive = true
        profilebutton.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor).isActive = true
        profilebutton.topAnchor.constraint(greaterThanOrEqualTo: self.topAnchor, constant: CGFloat(topPaddingProfileImage)) .isActive = true
        profilebutton.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor, constant: CGFloat(-bottomPaddingProfileImage)).isActive = true
        var notchPadding = 0
        if UIDevice.current.hasNotch{
            notchPadding = 12
        }
        profilebutton.centerYAnchor.constraint(equalTo: self.centerYAnchor, constant: CGFloat(notchPadding)).isActive = true
        profilebutton.widthAnchor.constraint(equalTo: profilebutton.heightAnchor, constant: 0).isActive = true
        profilebutton.heightAnchor.constraint(equalToConstant: 76).isActive = true
        let userInfo = ZohoCreatorAPIHandler.fetchMyUserInfoFromCache()
        let email = userInfo?.email ?? ""
        let userName = userInfo?.fullName ?? ""
        
        let leftPaddingOfHolderView = 20
        let rightPaddingOfHolderView = 10
        let topPaddingOfHolderView = topPaddingProfileImage + 0
        let bottomPaddingOfHolderView = 30
        
        
        let userNameAndIdHolderView = UIView()
        userNameAndIdHolderView.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(userNameAndIdHolderView)
        
        userNameAndIdHolderView.leftAnchor.constraint(equalTo: self.profilebutton.rightAnchor, constant: CGFloat(leftPaddingOfHolderView)).isActive = true
        userNameAndIdHolderView.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor, constant:CGFloat(-rightPaddingOfHolderView) ).isActive = true
        userNameAndIdHolderView.topAnchor.constraint(greaterThanOrEqualTo: self.topAnchor, constant: CGFloat(topPaddingOfHolderView)).isActive = true
        userNameAndIdHolderView.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor, constant: CGFloat(-bottomPaddingOfHolderView)).isActive = true
        userNameAndIdHolderView.centerYAnchor.constraint(equalTo: profilebutton.centerYAnchor, constant: 0).isActive = true
        
        let userNameLabel = UILabel()
        userNameLabel.text = userName
        userNameLabel.translatesAutoresizingMaskIntoConstraints = false
        userNameLabel.textColor = UIColor.white
        userNameLabel.font = UIFont.systemFont(ofSize: 20, weight: UIFont.Weight.bold)
        userNameAndIdHolderView.addSubview(userNameLabel)
        
        userNameLabel.leftAnchor.constraint(equalTo: userNameAndIdHolderView.leftAnchor, constant: 0).isActive = true
        userNameLabel.topAnchor.constraint(equalTo: userNameAndIdHolderView.topAnchor, constant: 0).isActive = true
        userNameLabel.rightAnchor.constraint(equalTo: userNameAndIdHolderView.rightAnchor, constant: 0).isActive = true
        
        
        let emailLabel = UILabel()
        emailLabel.text = email
        emailLabel.translatesAutoresizingMaskIntoConstraints = false
        emailLabel.textColor = UIColor(red: 210/255 , green: 214/255, blue: 220/255, alpha: 1)
        emailLabel.font = UIFont.boldSystemFont(ofSize: 15)
        userNameAndIdHolderView.addSubview(emailLabel)
        
        emailLabel.leftAnchor.constraint(equalTo: userNameAndIdHolderView.leftAnchor, constant: 0).isActive = true
        emailLabel.topAnchor.constraint(equalTo: userNameLabel.bottomAnchor, constant: 0).isActive = true
        emailLabel.rightAnchor.constraint(equalTo: userNameAndIdHolderView.rightAnchor).isActive = true
        emailLabel.bottomAnchor.constraint(equalTo: userNameAndIdHolderView.bottomAnchor).isActive = true
        
        
        self.setNeedsLayout()
        self.layoutIfNeeded()
        
        profilebutton.layer.cornerRadius = profilebutton.frame.size.width/2.0
        profilebutton.clipsToBounds = true
        
    }
    
    @objc func logoutAction() {
        let alert = UIAlertController(title: nil, message: "Are you sure you want to logout?", preferredStyle: .actionSheet)
        UIViewController.topViewController()?.present(alert, animated: true, completion:nil)
        alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
        alert.addAction(UIAlertAction(title: "Logout", style: .destructive, handler: { (action) in
            ZohoAuth.revokeAccessToken({ (error) in
                (UIApplication.shared.delegate as? AppDelegate)?.showInitialController()
            })
        }))
    }
    
    func showFieldInfo(withFieldName fieldDisplayName : String, andFieldDescription fieldDescription: String, buttonText: String?, withFormVC formVC : FormViewController) {
        let leftPaddingBackbutton = 20
        
        
        let topPaddingBackbutton = 50
        let bottomPaddingBackbutton = 10
        let  backButtonWidthOrHeight = 50.0
        self.formVC = formVC
        
        self.setBGColor()
        if let image = UIImage(named: "Back"){
            backButton.setImage(image, for: .normal)
        }
        backButton.contentMode = UIView.ContentMode.center
        backButton.backgroundColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.1)
        backButton.translatesAutoresizingMaskIntoConstraints = false
        backButton.layer.cornerRadius = CGFloat(backButtonWidthOrHeight/2)
        
        self.addSubview(backButton)
        
        backButton.leftAnchor.constraint(equalTo: self.leftAnchor, constant: CGFloat(leftPaddingBackbutton)).isActive = true
        backButton.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor).isActive = true
        backButton.topAnchor.constraint(equalTo: self.topAnchor, constant: CGFloat(topPaddingBackbutton)) .isActive = true
        backButton.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor, constant: CGFloat(-bottomPaddingBackbutton)).isActive = true
        backButton.widthAnchor.constraint(equalToConstant: CGFloat(backButtonWidthOrHeight)).isActive = true
        backButton.heightAnchor.constraint(equalToConstant: CGFloat(backButtonWidthOrHeight)).isActive = true
        
        backButton.addTarget(self, action: #selector(buttonTapped), for: .touchUpInside)
        
        
        fieldInfoLabel.text = fieldDisplayName
        fieldInfoLabel.textColor = UIColor.white
        fieldInfoLabel.font = UIFont.boldSystemFont(ofSize: 22)
        fieldInfoLabel.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(fieldInfoLabel)
        
        fieldInfoLabel.topAnchor.constraint(greaterThanOrEqualTo: self.centerYAnchor).isActive = true
        fieldInfoLabel.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor).isActive = true
        fieldInfoLabel.leftAnchor.constraint(equalTo: self.leftAnchor, constant: CGFloat(leftPaddingBackbutton)).isActive = true
        fieldInfoLabel.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor).isActive = true
        
        fieldDescriptionInfoLabel.text = fieldDescription
        fieldDescriptionInfoLabel.textColor = UIColor.white
        fieldDescriptionInfoLabel.font = UIFont.systemFont(ofSize: 14)
        fieldDescriptionInfoLabel.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(fieldDescriptionInfoLabel)
        
        fieldDescriptionInfoLabel.topAnchor.constraint(equalTo: fieldInfoLabel.bottomAnchor ,constant:5).isActive = true
        fieldDescriptionInfoLabel.bottomAnchor.constraint(equalTo: self.bottomAnchor , constant:CGFloat(-30)).isActive = true
        fieldDescriptionInfoLabel.leftAnchor.constraint(equalTo: self.leftAnchor, constant: CGFloat(leftPaddingBackbutton)).isActive = true
        fieldDescriptionInfoLabel.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor).isActive = true
        
        if let buttonTextStr = buttonText {
            
            nextButton.setTitle(buttonTextStr, for: .normal)
            nextButton.backgroundColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.1)
            nextButton.titleLabel?.font = UIFont.boldSystemFont(ofSize: 16)
            nextButton.translatesAutoresizingMaskIntoConstraints = false
            nextButton.layer.cornerRadius = 19
            nextButton.addTarget(self, action: #selector(nextButtonTapped), for: .touchUpInside)
            self.addSubview(nextButton)
            
            
            nextButton.topAnchor.constraint(greaterThanOrEqualTo: self.centerYAnchor).isActive = true
            nextButton.bottomAnchor.constraint(equalTo: self.bottomAnchor , constant:CGFloat(-30)).isActive = true
            nextButton.leftAnchor.constraint(greaterThanOrEqualTo: self.leftAnchor, constant: CGFloat(leftPaddingBackbutton)).isActive = true
            nextButton.rightAnchor.constraint(equalTo: self.rightAnchor, constant:CGFloat(-leftPaddingBackbutton)).isActive = true
            
            nextButton.widthAnchor.constraint(equalToConstant: 100).isActive = true
            nextButton.heightAnchor.constraint(equalToConstant: 40).isActive = true
        }
    }
    
    @objc func buttonTapped(){
        if let formVC = self.formVC {
            formVC.handleBackButtonAction()
        }
    }
    
    @objc func nextButtonTapped(){
        if let formVC = self.formVC {
            formVC.handleNextButtonAction()
        }
    }
}

extension UIDevice {
    var hasNotch: Bool {
        let bottom = UIApplication.shared.keyWindow?.safeAreaInsets.bottom ?? 0
        return bottom > 0
    }
}
