//
//  TimeAndLocationsChoicesLister.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 07/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import Foundation
import ZCCoreFramework

public enum ListerType {
    case location
    case time
}

class TimeAndLocationsChoicesLister: UIViewController {
    
    var choiceList:[Choice] = []
    var listerType: ListerType?
    var selectedChoice: Choice?
    
    weak var parentView: ShuttleView?
    
    
    static var timeAndLocationsReuseIds : String {
        return String(describing: self)
    }
    
    var choiceListTableView = UITableView ()
    var loader = UIActivityIndicatorView()
    
    
    override func viewWillAppear(_ animated: Bool) {
        self.navigationController?.setNavigationBarHidden(false, animated: false)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        self.navigationController?.setNavigationBarHidden(true, animated: false)
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.view?.backgroundColor = UIColor.brown
        
        self.view.addSubview(choiceListTableView)
        choiceListTableView.translatesAutoresizingMaskIntoConstraints = false
        choiceListTableView.delegate = self
        choiceListTableView.dataSource = self
        choiceListTableView.register(UITableViewCell.self, forCellReuseIdentifier: TimeAndLocationsChoicesLister.timeAndLocationsReuseIds)
        choiceListTableView.separatorStyle = .singleLine
        choiceListTableView.tableFooterView = UIView.init(frame: CGRect(x: 0, y: 0, width: 1, height: 1))
        
        choiceListTableView.leftAnchor.constraint(equalTo: self.view.leftAnchor, constant: 0).isActive = true
        choiceListTableView.topAnchor.constraint(equalTo: self.view.topAnchor, constant: 0).isActive = true
        choiceListTableView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: 0).isActive = true
        choiceListTableView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: 0).isActive = true
        choiceListTableView.reloadData()
        
        configureNaviagationBar()
    }
    
    func configureNaviagationBar () {
        
        var barTitle = "Select Choice"
        if( listerType == .location ){
            barTitle = "Select Available Locations"
        }else if( listerType == .time ){
            barTitle = "Select Available Time Slot"
        }
        self.navigationItem.title = barTitle
        self.navigationItem.backBarButtonItem = UIBarButtonItem(title: "back", style: .plain, target: nil, action: nil)
    }
}


extension TimeAndLocationsChoicesLister: UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return choiceList.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: TimeAndLocationsChoicesLister.timeAndLocationsReuseIds, for: indexPath) as UITableViewCell
        let choice = choiceList[indexPath.row]
        if(choice.key == selectedChoice?.key) {
            cell.accessoryType = UITableViewCell.AccessoryType.checkmark
        }
        cell.textLabel?.text = choice.key
        
        
        return cell
    }
}


extension TimeAndLocationsChoicesLister: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let selectedChoice = choiceList[indexPath.row]
        
        parentView!.selectedChoice(choice: selectedChoice,type: listerType!)
        self.navigationController?.popViewController(animated: true)
    }
}
