//
//  CollectionViewCell.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 05/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class CollectionViewCell: UICollectionViewCell {
    
    var textLabel: UILabel!
    var detailTextLabel: UILabel!
    var imageView: UIImageView!

    override init(frame: CGRect) {
        super.init(frame: frame)
        let imageYPadding = 30
        let imageWidth = 80
        let imageHeight = 80
        
        
        imageView = UIImageView.init()
        imageView.contentMode = UIView.ContentMode.center
        imageView.layer.borderWidth = 1
        imageView.layer.borderColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.05).cgColor
        imageView.layer.cornerRadius = 10
        imageView.translatesAutoresizingMaskIntoConstraints = false
        imageView.image = UIImage.init(named: "Supplies") ?? nil
        contentView.addSubview(imageView)
        
        contentView.addConstraints([
            NSLayoutConstraint.init(item: imageView, attribute: .left, relatedBy: .greaterThanOrEqual, toItem: contentView, attribute: .left, multiplier: 1, constant: 0),
            NSLayoutConstraint.init(item: imageView, attribute: .right, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .right, multiplier: 1, constant: 0),
            NSLayoutConstraint.init(item: imageView, attribute: .top, relatedBy: .equal, toItem: contentView, attribute: .top, multiplier: 1, constant: CGFloat(imageYPadding)),
            NSLayoutConstraint.init(item: imageView, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .bottom, multiplier: 1, constant: 0),
            NSLayoutConstraint.init(item: imageView, attribute: .width, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1, constant: CGFloat(imageWidth)),
            NSLayoutConstraint.init(item: imageView, attribute: .height, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1, constant: CGFloat(imageHeight)),
            NSLayoutConstraint.init(item: imageView, attribute: .centerX, relatedBy: .equal, toItem: contentView, attribute: .centerX, multiplier: 1, constant: 0),
            ])
        
        let textLabelFont = UIFont.boldSystemFont(ofSize: 16)
        let detailTextLabelFont =  UIFont.systemFont(ofSize: 13)


        textLabel = UILabel.init()
//        textLabel.backgroundColor = UIColor.red
        textLabel.textAlignment = .center
        textLabel.textColor = UIColor.black
        textLabel.font = textLabelFont
        textLabel.translatesAutoresizingMaskIntoConstraints = false
        contentView.addSubview(textLabel)
        
        let leftPaddingOfTextLabel = 20
        let rightPaddingOfTextLabel = leftPaddingOfTextLabel
        let topPaddingOfTextLabel = 15
        let topPaddingOfDetailTextLabel = 5
        let bottomPaddingOfDetailTextLabel = 0
        
        contentView.addConstraints([
            NSLayoutConstraint.init(item: textLabel, attribute: .left, relatedBy: .greaterThanOrEqual, toItem: contentView, attribute: .left, multiplier: 1, constant: CGFloat(leftPaddingOfTextLabel)),
            NSLayoutConstraint.init(item: textLabel, attribute: .right, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .right, multiplier: 1, constant: CGFloat(-rightPaddingOfTextLabel)),
            NSLayoutConstraint.init(item: textLabel, attribute: .top, relatedBy: .equal, toItem: imageView, attribute: .bottom, multiplier: 1, constant: CGFloat(topPaddingOfTextLabel)),
            NSLayoutConstraint.init(item: textLabel, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .bottom, multiplier: 1, constant: 0),
            NSLayoutConstraint.init(item: textLabel, attribute: .centerX, relatedBy: .equal, toItem: contentView, attribute: .centerX, multiplier: 1, constant: 0),
            ])

        detailTextLabel = UILabel.init()
        detailTextLabel.textAlignment = .center
        detailTextLabel.textColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.5)
        detailTextLabel.font = detailTextLabelFont
//        detailTextLabel.backgroundColor = UIColor.yellow
        detailTextLabel.numberOfLines = 2
        detailTextLabel.translatesAutoresizingMaskIntoConstraints = false
        contentView.addSubview(detailTextLabel)
        
        contentView.addConstraints([
            NSLayoutConstraint.init(item: detailTextLabel, attribute: .left, relatedBy: .greaterThanOrEqual, toItem: contentView, attribute: .left, multiplier: 1, constant: CGFloat(leftPaddingOfTextLabel)),
            NSLayoutConstraint.init(item: detailTextLabel, attribute: .right, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .right, multiplier: 1, constant: CGFloat(-rightPaddingOfTextLabel)),
            NSLayoutConstraint.init(item: detailTextLabel, attribute: .top, relatedBy: .equal, toItem: textLabel, attribute: .bottom, multiplier: 1, constant: CGFloat(topPaddingOfDetailTextLabel)),
            NSLayoutConstraint.init(item: detailTextLabel, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: contentView, attribute: .bottom, multiplier: 1, constant: CGFloat(bottomPaddingOfDetailTextLabel)),
            NSLayoutConstraint.init(item: detailTextLabel, attribute: .centerX, relatedBy: .equal, toItem: contentView, attribute: .centerX, multiplier: 1, constant: 0),
            ])
        
        self.backgroundColor = UIColor.white
        self.layer.cornerRadius = 10
        
        self.layer.masksToBounds = false
        self.layer.shadowColor = UIColor.black.cgColor
        self.layer.shadowOpacity = 0.1
        self.layer.shadowOffset = CGSize(width: 0, height: 4)
        self.layer.shadowRadius = 10
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    func configureCell(forSection section: Section) {
        self.textLabel.text = section.sectionName
        self.configureImageAndDescription(forSection: section)
    }
    
    func configureImageAndDescription(forSection section : Section)  {
        let imageName = section.sectionLinkName
        var image : UIImage? = UIImage.init()

        switch imageName {
        case "Supplies":
            image = UIImage.init(named: "Supplies") ?? nil
            detailTextLabel.text = "Get your preferred goods"
        case "Shuttles":
            image = UIImage.init(named: "Shuttles") ?? nil
            detailTextLabel.text =  "Book your shuttle"
        case "Canteens":
            image = UIImage.init(named: "Canteens") ?? nil
            detailTextLabel.text = "Get today's menu"

        case "Complaints":
            image = UIImage.init(named: "Complaints") ?? nil
                detailTextLabel.text = "Raise a complaint on our services"

        default: break

        }
          self.imageView.image = image
        }
    
    }
