//
//  FieldCollectionViewCell.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 06/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class FieldCollectionViewCell: UICollectionViewCell {
    
    var fieldImageView: UIImageView = UIImageView.init()
    var textLabel: UILabel = UILabel.init()
    var cellImageName : String?
    var selectedTickImgView : UIImageView = UIImageView()
    
    
    override init(frame: CGRect) {
        
        super.init(frame: frame)
        
    }
    
    required init?(coder aDecoder: NSCoder) {
        
        fatalError("init(coder:) has not been implemented")
        
    }
    
    
    
    func configureCell(forChoice choice:Choice) {
        
        self.layer.cornerRadius = 10
        
        let imageTopPadding = 10
        let imageViewWidthOrHeight = 70.0
        
        fieldImageView.contentMode = UIView.ContentMode.center
        fieldImageView.translatesAutoresizingMaskIntoConstraints = false
        //        fieldImageView.backgroundColor = UIColor.purple
        
        cellImageName = SuppliesFormHelper.getImage(forChoiceKey: choice.key)
        fieldImageView.image = UIImage.init(named: cellImageName ?? "")
        fieldImageView.layer.cornerRadius = CGFloat(imageViewWidthOrHeight/2)
        fieldImageView.layer.borderColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.05).cgColor
        fieldImageView.layer.borderWidth = 2
        
        contentView.addSubview(fieldImageView)
        
        
        fieldImageView.topAnchor.constraint(equalTo: contentView.topAnchor, constant: CGFloat(imageTopPadding)).isActive = true
        fieldImageView.bottomAnchor.constraint(lessThanOrEqualTo: contentView.bottomAnchor).isActive = true
        fieldImageView.centerXAnchor.constraint(equalTo: contentView.centerXAnchor, constant: 0).isActive = true
        fieldImageView.widthAnchor.constraint(equalToConstant: CGFloat (imageViewWidthOrHeight) ).isActive = true
        fieldImageView.heightAnchor.constraint(equalToConstant: CGFloat(imageViewWidthOrHeight)).isActive = true
        
        
        selectedTickImgView.contentMode = UIView.ContentMode.center
        selectedTickImgView.translatesAutoresizingMaskIntoConstraints = false
        selectedTickImgView.image = UIImage.init(named:"SelectedTick")
        selectedTickImgView.isHidden = true
        contentView.addSubview(selectedTickImgView)
        
        selectedTickImgView.rightAnchor.constraint(equalTo: fieldImageView.rightAnchor, constant: 0).isActive = true
        selectedTickImgView.bottomAnchor.constraint(equalTo: fieldImageView.bottomAnchor, constant: 0).isActive = true
        selectedTickImgView.leftAnchor.constraint(greaterThanOrEqualTo: fieldImageView.leftAnchor, constant: 0).isActive = true
        selectedTickImgView.topAnchor.constraint(greaterThanOrEqualTo: fieldImageView.topAnchor, constant: 0).isActive = true
        
        
        textLabel.font = UIFont.systemFont(ofSize: 13)
        textLabel.text = choice.value
        textLabel.translatesAutoresizingMaskIntoConstraints = false
        textLabel.textAlignment = .center
        textLabel.textColor = UIColor(red: 127/255, green: 127/255, blue: 127/255, alpha: 1)
        contentView.addSubview(textLabel)
        
        let topPaddingOfFieldText = 10
        let bottomPaddingOfFieldText = topPaddingOfFieldText
        
        textLabel.topAnchor.constraint(equalTo: fieldImageView.bottomAnchor, constant: CGFloat(topPaddingOfFieldText)).isActive = true
        textLabel.bottomAnchor.constraint(lessThanOrEqualTo: contentView.bottomAnchor, constant: CGFloat(-bottomPaddingOfFieldText)).isActive = true
        textLabel.centerXAnchor.constraint(equalTo: contentView.centerXAnchor, constant: 0).isActive = true
        textLabel.leftAnchor.constraint(equalTo: contentView.leftAnchor, constant: 0).isActive = true
        textLabel.rightAnchor.constraint(equalTo: contentView.rightAnchor, constant: 0).isActive = true
        
    }
    
    func setSelectedImage() {
        if var selectedImageName = cellImageName {
            selectedImageName += "Selected"
            fieldImageView.image = UIImage.init(named: selectedImageName)
            selectedTickImgView.isHidden = false
        }
    }
    
    func setUnSelectedImage() {
        if let imageName = cellImageName {
            fieldImageView.image = UIImage.init(named: imageName)
            selectedTickImgView.isHidden = true
        }
    }
    
    
}

class SuppliesFormHelper{
    
    static func getImage(forChoiceKey choiceKey : String) -> String? {
        
        var imageName : String?
        
        switch choiceKey {
        case "Cupboard":
            imageName = "Cupboard"
        case "Footrest":
            imageName = "FootRest"
        case "Back Buddy":
            imageName = "BackBuddy"
        case "White board":
            imageName = "WhiteBoard"
        case "Bean Bag":
            imageName = "BeanBag"
        case "New Cupboard":
            imageName = "NewCupBoard"
        case "Name Board":
            imageName = "NameBoard"
        case "Laptop Stand":
            imageName = "LaptopStand"
        case "Chair":
            imageName = "Chair"
        case "Place Change":
            imageName = "PlaceChange"
        default: break
            
        }
        return imageName ?? ""
    }
    static func getLocationFieldDescriptionInfo() -> String
    {
        return "Enter your seating location"
    }
    
    static func getSupplyFieldInfo () -> (title : String , description : String , buttonTitle : String)
    {
        let title = "Supplies"
        let description = "Pick the goods which you need"
        let nextButtonTitle = "Next"
        
        return (title,description,nextButtonTitle)
    }
    static func getOrderLabelTitle() -> String
    {
        return "Order Placed Successfully"
    }
    static func getOrderDescription() -> String
    {
        return "You'll receive below supplies shortly on"
    }
}
