//
//  SuppliesFormLocationInputView.swift
//  Zoho Creator SDK Demo
//
//  Created by ZOHO corporation  on 07/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class SuppliesRequestedCell: UICollectionViewCell {
    
    lazy var selectedCellImageView : UIImageView = {
        let imageView = UIImageView()
        imageView.image = UIImage.init(named: "done") ?? nil
        imageView.contentMode = .center
        imageView.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(imageView)
        NSLayoutConstraint.activate([
            imageView.centerYAnchor.constraint(equalTo: self.centerYAnchor, constant:0),
            imageView.leftAnchor.constraint(equalTo: self.leftAnchor, constant: 0),
            ])
        
        return imageView
    }()
    
    lazy var textLabel : UILabel = {
        let textLabel = UILabel()
        textLabel.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(textLabel)
        NSLayoutConstraint.activate([
            textLabel.centerYAnchor.constraint(equalTo: self.centerYAnchor, constant:0),
            textLabel.leftAnchor.constraint(equalTo: selectedCellImageView.rightAnchor, constant: 0),
            textLabel.rightAnchor.constraint(equalTo: self.rightAnchor, constant: -15),
            ])
        textLabel.font = UIFont.systemFont(ofSize: 14)
        //        textLabel.isHidden = true
        return textLabel
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.contentView.layer.borderColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.05).cgColor
        self.contentView.layer.borderWidth = 2
        self.contentView.layer.cornerRadius = 20
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

class SuppliesFormLocationInputView: UIView {
    
    var suppliesView:SuppliesView?
    let activityIndicator = UIActivityIndicatorView.init(frame: CGRect.init(x: 0, y: 0, width: 25, height: 25))
    let textField = UITextField()
    let fieldTitleLabel = UILabel()
    let fieldDescriptionLabel = UILabel()
    let cancelButton = UIButton()
    let submitButton = GradientButton()
    var currentField:FieldProtocol?
    var suppliesChoiceField:FieldProtocol?
    
    let topPaddingFieldTitle = 20
    let leftPaddingFieldTitle = 20
    let rightPaddingFieldTitle = 10
    let descriptionLabelTopPadding = 10
    var keyBoardFrame:CGFloat = 0.0
    var isKeyBoardVisible = false
    
    static var reuseIDForCells = "reuseIDS"
    
    override func layoutSubviews() {
        super.layoutSubviews()
        self.roundCorners(corners: [.topLeft, .topRight], radius: 10)
    }
    func configureSuppliesFormInputView(withForm form:Form)
    {
        let totalFields = form.fields.count
        if  totalFields > 0 {
            for index in 0..<totalFields
            {
                let field = form.fields[index]
                if field.linkName == "Seating_Location"
                {
                    currentField = field
                    self.configureView(forFieldDisplayName: field.displayName, fieldDescriptionName: SuppliesFormHelper.getLocationFieldDescriptionInfo())
                    break
                }
            }
        }
    }
    
    private func configureView(forFieldDisplayName fieldDisplayStr : String , fieldDescriptionName fieldDescriptionStr : String)  {
        self.backgroundColor = UIColor.white
        
        NotificationCenter.default.removeObserver(self)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        fieldTitleLabel.text = fieldDisplayStr
        fieldTitleLabel.font = UIFont.boldSystemFont(ofSize: 18)
        fieldTitleLabel.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(fieldTitleLabel)
        
        fieldTitleLabel.topAnchor.constraint(equalTo: self.topAnchor, constant:CGFloat(topPaddingFieldTitle)).isActive = true
        fieldTitleLabel.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor).isActive = true
        fieldTitleLabel.leftAnchor.constraint(equalTo: self.leftAnchor, constant:CGFloat(leftPaddingFieldTitle)).isActive = true
        fieldTitleLabel.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor, constant:CGFloat(-rightPaddingFieldTitle)).isActive = true
        
        
        fieldDescriptionLabel.text = fieldDescriptionStr
        fieldDescriptionLabel.font = UIFont.systemFont(ofSize: 14)
        fieldDescriptionLabel.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(fieldDescriptionLabel)
        
        fieldDescriptionLabel.topAnchor.constraint(equalTo: fieldTitleLabel.bottomAnchor, constant:CGFloat(descriptionLabelTopPadding)).isActive = true
        fieldDescriptionLabel.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor).isActive = true
        fieldDescriptionLabel.leftAnchor.constraint(equalTo: self.leftAnchor, constant:CGFloat(leftPaddingFieldTitle)).isActive = true
        fieldDescriptionLabel.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor, constant:CGFloat(-rightPaddingFieldTitle)).isActive = true
        
        textField.font = UIFont.boldSystemFont(ofSize: 35)
        textField.autocorrectionType = UITextAutocorrectionType.no
        textField.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(textField)
        
        let textFieldTopPadding = 10
        let textFieldBottomPadding = 10
        
        textField.topAnchor.constraint(equalTo: fieldDescriptionLabel.bottomAnchor, constant:CGFloat(textFieldTopPadding)).isActive = true
        textField.bottomAnchor.constraint(lessThanOrEqualTo: self.bottomAnchor, constant: CGFloat(-textFieldBottomPadding)).isActive = true
        textField.leftAnchor.constraint(equalTo: self.leftAnchor, constant:CGFloat(leftPaddingFieldTitle)).isActive = true
        textField.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor, constant:CGFloat(-rightPaddingFieldTitle)).isActive = true
        
        
        cancelButton.translatesAutoresizingMaskIntoConstraints = false
        cancelButton.setTitle("Cancel", for: .normal)
        cancelButton.titleLabel?.font = UIFont.systemFont(ofSize: 16)
        cancelButton.setTitleColor(UIColor.init(red: 201.0/255.0, green: 135.0/255.0, blue: 236.0/255.0, alpha: 1), for: .normal)
        cancelButton.addTarget(self, action: #selector(cancelClicked), for: .touchUpInside)
        self.addSubview(cancelButton)
        
        let topPaddingOfCancelButton = 20
        let bottomPaddingOfCancelButton = 20
        
        cancelButton.topAnchor.constraint(equalTo: textField.bottomAnchor, constant:CGFloat(topPaddingOfCancelButton)).isActive = true
        cancelButton.bottomAnchor.constraint(equalTo: self.bottomAnchor, constant: CGFloat(-bottomPaddingOfCancelButton)).isActive = true
        cancelButton.leftAnchor.constraint(equalTo: self.leftAnchor, constant:CGFloat(leftPaddingFieldTitle)).isActive = true
        cancelButton.rightAnchor.constraint(lessThanOrEqualTo: self.rightAnchor, constant:CGFloat(-rightPaddingFieldTitle)).isActive = true
        
        submitButton.translatesAutoresizingMaskIntoConstraints = false
        submitButton.titleLabel?.textColor = UIColor.white
        submitButton.titleLabel?.font = UIFont.systemFont(ofSize: 15)
        submitButton.setTitleColor(.white, for: .normal)
        submitButton.setTitle("Submit", for: .normal)
        submitButton.contentMode = .center
        submitButton.layer.cornerRadius = 20
        submitButton.addTarget(self, action: #selector(submitClicked(sender:)), for: .touchUpInside)
        self.addSubview(submitButton)
        
        submitButton.topAnchor.constraint(equalTo: textField.bottomAnchor, constant:CGFloat(topPaddingOfCancelButton)).isActive = true
        submitButton.bottomAnchor.constraint(equalTo: self.bottomAnchor, constant: CGFloat(-bottomPaddingOfCancelButton)).isActive = true
        submitButton.leftAnchor.constraint(greaterThanOrEqualTo: self.leftAnchor, constant:CGFloat(leftPaddingFieldTitle)).isActive = true
        submitButton.rightAnchor.constraint(equalTo: self.rightAnchor, constant:CGFloat(-rightPaddingFieldTitle)).isActive = true
        let submitButtonHeight : CGFloat = 42
        submitButton.widthAnchor.constraint(equalToConstant: 120).isActive = true
        submitButton.heightAnchor.constraint(equalToConstant: submitButtonHeight).isActive = true
        
        self.setNeedsLayout()
        self.layoutIfNeeded()
        
        submitButton.gradientLayer.colors = [UIColor(red: 201/255.0, green: 135/255.0, blue: 236/255.0, alpha: 1/1.0).cgColor,UIColor(red: 251/255.0, green: 113/255.0, blue: 192/255.0, alpha: 1/1.0).cgColor]
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.3) {
            self.textField.becomeFirstResponder()
        }
    }
    func configureAfterSuccess(withSelectedSupplies supplyArray: [Choice]) {
        let suppliesSuccessInfoView = SuppliesSuccessView()
        suppliesSuccessInfoView.translatesAutoresizingMaskIntoConstraints = false
        suppliesSuccessInfoView.suppliesView = self.suppliesView
        suppliesSuccessInfoView.backgroundColor = UIColor.white
        if let blurView = self.superview
        {
            blurView.addSubview(suppliesSuccessInfoView)
            suppliesSuccessInfoView.topAnchor.constraint(greaterThanOrEqualTo: blurView.topAnchor, constant: 0).isActive = true
            suppliesSuccessInfoView.bottomAnchor.constraint(equalTo: blurView.bottomAnchor, constant: 0).isActive = true
            suppliesSuccessInfoView.leftAnchor.constraint(equalTo: blurView.leftAnchor, constant: 0).isActive = true
            suppliesSuccessInfoView.rightAnchor.constraint(equalTo: blurView.rightAnchor, constant: 0).isActive = true
        }
        
        suppliesSuccessInfoView.configureView(withLocation: (textField.text), andSelectedSupplies: (suppliesChoiceField as? CheckboxField)?.fieldValue.selectedChoices)
        
        textField.resignFirstResponder()
        textField.text = ""
    }
    
    @objc func keyboardWillShow(notification: NSNotification) {
        
        if !isKeyBoardVisible {
            if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
                keyBoardFrame = keyboardSize.height
                isKeyBoardVisible = true
                self.frame.origin.y -= (keyboardSize.height  )
            }
        }
        
    }
    
    @objc func keyboardWillHide(notification: NSNotification) {
        if isKeyBoardVisible
        {
            isKeyBoardVisible = false
            keyBoardFrame = 0
            self.frame.origin.y -=  (keyBoardFrame)
        }
    }
    
    @objc func cancelClicked(){
        self.suppliesView?.cancelClicked()
    }
    @objc func submitClicked(sender : UIButton){
        if let enteredText = textField.text  {
            if enteredText.count > 0{
                var field = ( currentField as! SingleLineFieldProtocol)
                field.fieldValue.value = enteredText
                
                self.addLoader(inButton: sender)
                self.suppliesView?.submitClicked(withLocationField: field)
            }
        }
    }
    
    func addLoader(inButton button:UIButton){
        
        button.setTitle("", for: .normal)
        activityIndicator.center = button.center
        activityIndicator.style = .white
        self.addSubview(activityIndicator)
        
        activityIndicator.startAnimating()
        
    }
    func removeLoader(inButton button:UIButton){
        button.setTitle("Submit", for: .normal)
        activityIndicator.stopAnimating()
        activityIndicator.removeFromSuperview()
        
    }
}

extension UIView {
    func roundCorners(corners: UIRectCorner, radius: CGFloat) {
        let path = UIBezierPath(roundedRect: bounds, byRoundingCorners: corners, cornerRadii: CGSize(width: radius, height: radius))
        let mask = CAShapeLayer()
        mask.path = path.cgPath
        layer.mask = mask
    }
}

