//
//  SuppliesSuccessView.swift
//  ZohoCreatorSDKDemo
//
//  Created by ZOHO corporation  on 12/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class SuppliesSuccessView: UIView,UICollectionViewDelegateFlowLayout,UICollectionViewDataSource, UICollectionViewDelegate {
    
    let topPaddingFieldTitle = 20
    let leftPaddingFieldTitle = 20
    let rightPaddingFieldTitle = 10
    let descriptionLabelTopPadding = 10
    var keyBoardFrame:CGFloat = 0.0
    private let supplyViewCellHeight : CGFloat = 40
    
    var enteredSeatingLocation : String = ""
    var selectedChoicesOfSupplyView : [Choice] = []
    
    lazy var orderedItemsCollectionView: UICollectionView = {
        
        let layout: UICollectionViewFlowLayout = SuppliesFormLocationInputFlowLayout()
        layout.sectionInset = UIEdgeInsets(top: 20, left: 10, bottom: 10, right: 10)
        layout.minimumInteritemSpacing = 10
        layout.minimumLineSpacing = 10
        
        if #available(iOS 11.0, *) {
            layout.sectionInsetReference = .fromSafeArea
        }
        return UICollectionView(frame: CGRect.zero, collectionViewLayout: layout)
    }()
    let closeButton = UIButton()
    var suppliesView:SuppliesView?
    
    func configureView(withLocation seatingLocation : String?, andSelectedSupplies selectedChoices : [Choice]?  ) {
        
        if let seatingLocation = seatingLocation , let selectedChoices = selectedChoices
        {
            enteredSeatingLocation = seatingLocation
            selectedChoicesOfSupplyView = selectedChoices
            
            let orderLabel = UILabel()
            orderLabel.text = SuppliesFormHelper.getOrderLabelTitle()
            orderLabel.font = UIFont.boldSystemFont(ofSize: 18)
            orderLabel.translatesAutoresizingMaskIntoConstraints = false
            self.addSubview(orderLabel)
            self.addConstraints([
                NSLayoutConstraint(item: orderLabel, attribute: .top, relatedBy: .equal, toItem: self, attribute: .top, multiplier: 1, constant: CGFloat(topPaddingFieldTitle)),
                NSLayoutConstraint(item: orderLabel, attribute: .left, relatedBy: .equal, toItem: self, attribute: .left, multiplier: 1, constant: CGFloat(leftPaddingFieldTitle)),
                NSLayoutConstraint(item: orderLabel, attribute: .right, relatedBy: .lessThanOrEqual, toItem: self, attribute: .right, multiplier: 1, constant: CGFloat(-rightPaddingFieldTitle)),
                NSLayoutConstraint(item: orderLabel, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: self, attribute: .bottom, multiplier: 1, constant: CGFloat(-topPaddingFieldTitle))
                ])
            
            let orderDescriptionLabel = UILabel()
            orderDescriptionLabel.text = "\(SuppliesFormHelper.getOrderDescription()) " + "\(seatingLocation)"
            orderDescriptionLabel.font = UIFont.systemFont(ofSize: 14)
            orderDescriptionLabel.translatesAutoresizingMaskIntoConstraints = false
            self.addSubview(orderDescriptionLabel)
            self.addConstraints([
                NSLayoutConstraint(item: orderDescriptionLabel, attribute: .top, relatedBy: .equal, toItem: orderLabel, attribute: .bottom, multiplier: 1, constant: CGFloat(descriptionLabelTopPadding)),
                NSLayoutConstraint(item: orderDescriptionLabel, attribute: .left, relatedBy: .equal, toItem: self, attribute: .left, multiplier: 1, constant: CGFloat(leftPaddingFieldTitle)),
                NSLayoutConstraint(item: orderDescriptionLabel, attribute: .right, relatedBy: .lessThanOrEqual, toItem: self, attribute: .right, multiplier: 1, constant: CGFloat(-rightPaddingFieldTitle)),
                NSLayoutConstraint(item: orderDescriptionLabel, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: self, attribute: .bottom, multiplier: 1, constant: CGFloat(-topPaddingFieldTitle))
                ])
            
            orderedItemsCollectionView.delegate = self
            orderedItemsCollectionView.dataSource = self
            orderedItemsCollectionView.backgroundColor = UIColor.init(red: 252.0/255.0, green: 252.0/255.0, blue: 252.0/255.0, alpha: 1)
            orderedItemsCollectionView.translatesAutoresizingMaskIntoConstraints = false
            self.addSubview(orderedItemsCollectionView)
            
            orderedItemsCollectionView.register(SuppliesRequestedCell.self, forCellWithReuseIdentifier: SuppliesFormLocationInputView.reuseIDForCells)
            
            let collectionViewTopPadding = 10
            let orderedItemsCollectionHeight : CGFloat = getHeightForSelectedFoodView()
            
            self.addConstraints([
                NSLayoutConstraint.init(item: orderedItemsCollectionView, attribute: .top, relatedBy: .equal, toItem: orderDescriptionLabel, attribute: .bottom, multiplier: 1.0, constant: CGFloat(collectionViewTopPadding)),
                NSLayoutConstraint.init(item: orderedItemsCollectionView, attribute: .left, relatedBy: .equal, toItem: self, attribute: .left, multiplier: 1.0, constant: CGFloat(leftPaddingFieldTitle)),
                NSLayoutConstraint.init(item: orderedItemsCollectionView, attribute: .right, relatedBy: .equal, toItem: self, attribute: .right, multiplier: 1.0, constant: CGFloat(-rightPaddingFieldTitle)),
                NSLayoutConstraint.init(item: orderedItemsCollectionView, attribute: .bottom, relatedBy: .lessThanOrEqual, toItem: self, attribute: .bottom, multiplier: 1.0, constant: 0),
                NSLayoutConstraint.init(item: orderedItemsCollectionView, attribute: .height, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1, constant: orderedItemsCollectionHeight)
                ])
            closeButton.setTitle("Close", for: .normal)
            closeButton.translatesAutoresizingMaskIntoConstraints = false
            closeButton.setTitleColor(UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.5), for: .normal)
            closeButton.addTarget(self, action: #selector(closeClicked), for: .touchUpInside)
            self.addSubview(closeButton)
            
            let rightPaddingCloseButton = 10
            let leftPaddingCloseButton = rightPaddingCloseButton
            let bottomPaddingCloseButton = 50
            let topPaddingCloseButtonFromColectionView = 10
            
            self.addConstraints([
                NSLayoutConstraint(item: closeButton, attribute: .right, relatedBy: .equal, toItem: self, attribute: .right, multiplier: 1, constant: CGFloat(-rightPaddingCloseButton)),
                NSLayoutConstraint(item: closeButton, attribute: .left, relatedBy: .greaterThanOrEqual, toItem: self, attribute: .left, multiplier: 1, constant: CGFloat(leftPaddingCloseButton)),
                NSLayoutConstraint(item: closeButton, attribute: .top, relatedBy: .equal, toItem: orderedItemsCollectionView, attribute: .bottom, multiplier: 1, constant: CGFloat(topPaddingCloseButtonFromColectionView)),
                NSLayoutConstraint(item: closeButton, attribute: .bottom, relatedBy: .equal, toItem: self, attribute: .bottom, multiplier: 1, constant: CGFloat(-bottomPaddingCloseButton)),
                NSLayoutConstraint.init(item: closeButton, attribute: .width, relatedBy: .equal, toItem: nil, attribute: .notAnAttribute, multiplier: 1, constant: 100)
                ])
        }
    }
    override func layoutSubviews() {
        super.layoutSubviews()
        self.roundCorners(corners: [.topLeft, .topRight], radius: 10)
    }
    @objc func cancelClicked(){
        self.suppliesView?.cancelClicked()
    }
    @objc func closeClicked(){
        self.suppliesView?.resetSuplyForm()
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return selectedChoicesOfSupplyView.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        if let collectionViewCell = collectionView.dequeueReusableCell(withReuseIdentifier: SuppliesFormLocationInputView.reuseIDForCells, for: indexPath) as? SuppliesRequestedCell {
            collectionViewCell.textLabel.text = selectedChoicesOfSupplyView[indexPath.row].value
            return collectionViewCell
        }
        
        return UICollectionViewCell()
    }
    
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        sizeForItemAt indexPath: IndexPath) -> CGSize {
        let value = selectedChoicesOfSupplyView[indexPath.row].value
        let width = value.size(withAttributes: [NSAttributedString.Key.font: UIFont.systemFont(ofSize: 14.0)]).width
        
        let imageWidth:CGFloat = (UIImage.init(named: "done")?.size.width)!
        let imageLeftPadding:CGFloat = 10
        let paddingBetweenImgAndText:CGFloat = 10
        let textRightPadding:CGFloat = 10
        let totalWidth =  width + imageWidth + imageLeftPadding + paddingBetweenImgAndText + textRightPadding + 10
        return CGSize(width: totalWidth, height: 40)
    }
    
    
    func getHeightForSelectedFoodView( ) -> CGFloat{
        let foodViewCalculatedHeight = calculateHeightForSelectedFoodView()
        var minimumHeight : CGFloat = 100
        
        if(foodViewCalculatedHeight > minimumHeight) {
            minimumHeight = foodViewCalculatedHeight
        }
        return minimumHeight
    }
    func  calculateHeightForSelectedFoodView() -> CGFloat{
        let screenWidth = UIScreen.main.bounds.size.width
        
        var totalCellsWidth : CGFloat = 0
        for choice in selectedChoicesOfSupplyView{
            
            let value = choice.value
            let width = value.size(withAttributes: [NSAttributedString.Key.font: UIFont.systemFont(ofSize: 14.0)]).width
            let imageWidth:CGFloat = (UIImage.init(named: "done")?.size.width)!
            let imageLeftPadding:CGFloat = 10
            let paddingBetweenImgAndText:CGFloat = 10
            let textRightPadding:CGFloat = 10
            let cellWidth =  width + imageWidth + imageLeftPadding + paddingBetweenImgAndText + textRightPadding + 10
            
            totalCellsWidth = totalCellsWidth + cellWidth
        }
        let selectedFoodsViewHeaderSpacing : CGFloat = 40
        
        if(totalCellsWidth > screenWidth) {
            var numberOfRows = NSInteger(totalCellsWidth / screenWidth)
            let reminder = NSInteger(totalCellsWidth) % NSInteger(screenWidth)
            if(reminder > 0){
                numberOfRows = numberOfRows + 1
            }
            let cellMinimumLineSpacing : CGFloat = 10
            let selectedFoodItemsHeaderHeight : CGFloat = 20
            
            return ( supplyViewCellHeight + cellMinimumLineSpacing) * CGFloat(numberOfRows) + selectedFoodItemsHeaderHeight + selectedFoodsViewHeaderSpacing
        }
        return 100
    }
}

