//
//  SuppliesView.swift
//  ZohoCreatorSDKDemo
//
//  Created by ZOHO corporation  on 08/02/19.
//  Copyright © 2019 ZOHO corporation . All rights reserved.
//

import UIKit
import ZCCoreFramework

class SuppliesView: UIView , UICollectionViewDelegateFlowLayout,UICollectionViewDataSource{
    lazy var fieldsCollectionView : UICollectionView = {
        let layout: UICollectionViewFlowLayout = UICollectionViewFlowLayout()
        layout.sectionInset = UIEdgeInsets(top: 20, left: 10, bottom: 10, right: 10)
        layout.minimumInteritemSpacing = 10
        layout.minimumLineSpacing = 0
        
        if #available(iOS 11.0, *) {
            layout.sectionInsetReference = .fromSafeArea
        }
        return UICollectionView(frame: CGRect.zero, collectionViewLayout: layout)
    }()
    static var reuseIDForFields = "reuse"
    var currentForm : Form?
    var currentChoiceField : FieldProtocol?
    var selectedChoices:[Choice] = []
    var formVC :FormViewController?
    let locationInputView = SuppliesFormLocationInputView()
    var isSupplyViewFormSubmitted = false
    
    
    func configureSuppliesView(withForm form:Form) {
        
        isSupplyViewFormSubmitted = false
        currentForm = form
        
        if let totalFields = currentForm?.fields.count {
            for index in 0..<totalFields
            {
                let field = currentForm?.fields[index]
                if field?.linkName == "Select_Supplies"
                {
                    currentChoiceField = field
                    fieldsCollectionView.delegate = self
                    fieldsCollectionView.dataSource = self
                    fieldsCollectionView.backgroundColor = UIColor.init(red: 252.0/255.0, green: 252.0/255.0, blue: 252.0/255.0, alpha: 1)
                    fieldsCollectionView.translatesAutoresizingMaskIntoConstraints = false
                    self.addSubview(fieldsCollectionView)
                    
                    fieldsCollectionView.register(FieldCollectionViewCell.self, forCellWithReuseIdentifier: SuppliesView.reuseIDForFields)
                    
                    fieldsCollectionView.leftAnchor.constraint(equalTo: self.leftAnchor, constant: 0).isActive = true
                    fieldsCollectionView.topAnchor.constraint(equalTo: self.topAnchor, constant: 0).isActive = true
                    fieldsCollectionView.rightAnchor.constraint(equalTo: self.rightAnchor, constant: 0).isActive = true
                    fieldsCollectionView.bottomAnchor.constraint(equalTo: self.bottomAnchor, constant: 0).isActive = true
                    
                    updateProfileNextButtonAlpha()
                    fieldsCollectionView.reloadData()
                }
            }
        }
    }
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return (currentChoiceField as? MultiSelectChoiceFieldProtocol )?.fieldValue.choices.count ?? 0
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView
            .dequeueReusableCell(withReuseIdentifier: SuppliesView.reuseIDForFields, for: indexPath) as! FieldCollectionViewCell
        if let currentChoice =  (currentChoiceField as? MultiSelectChoiceFieldProtocol )?.fieldValue.choices[indexPath.row]{
            cell.configureCell(forChoice: currentChoice )
        }
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        return CGSize(width: floor((collectionView.bounds.width - 40) / 3), height: 120)
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        let selectedCell = collectionView.cellForItem(at: indexPath) as? FieldCollectionViewCell
        if let selectedChoice = (currentChoiceField as? MultiSelectChoiceFieldProtocol )?.fieldValue.choices[indexPath.row] {
            
            if selectedChoices.contains(selectedChoice){
                selectedChoices = self.removeChoice(fromArray: selectedChoices, removingChoice: selectedChoice) as! [Choice]
                selectedCell!.setUnSelectedImage()
            }
            else
            {
                selectedChoices.append(selectedChoice)
                selectedCell!.setSelectedImage()
            }
        }
        updateProfileNextButtonAlpha()
    }
    
    func removeChoice(fromArray : Array<Any>, removingChoice : Choice) -> [Any]? {
        var indexOfChoice = 0
        var returningArray = fromArray
        
        for index in 0..<fromArray.count {
            let choice = fromArray[index] as! Choice
            if choice.key == removingChoice.key{
                indexOfChoice = index
                break
            }
        }
        
        returningArray.remove(at: indexOfChoice)
        return returningArray
    }
    func getSelectedChoices() -> [Choice] {
        return selectedChoices
    }
    
    func updateProfileNextButtonAlpha() {
        if  self.getSelectedChoices().count > 0{
            formVC?.profileHolderView.nextButton.alpha = 1
        } else {
              formVC?.profileHolderView.nextButton.alpha = 0.3
        }
    }
    
    func handleNextButtonAction() {
        if  self.getSelectedChoices().count > 0{
            if let formVCc = self.formVC {
                let blurView = formVCc.addBlurView()
                
                locationInputView.translatesAutoresizingMaskIntoConstraints = false
                if let currentForm = self.currentForm{
                    locationInputView.configureSuppliesFormInputView(withForm:currentForm)
                    locationInputView.suppliesView = self
                }
                
                var suppliesChoiceField = (currentChoiceField as? MultiSelectChoiceFieldProtocol )
                suppliesChoiceField?.fieldValue.selectedChoices = selectedChoices
                self.currentChoiceField = suppliesChoiceField
                
                locationInputView.suppliesChoiceField = self.currentChoiceField
                
                locationInputView.suppliesChoiceField = currentChoiceField
                blurView.addSubview(locationInputView)
                
                locationInputView.topAnchor.constraint(greaterThanOrEqualTo: blurView.topAnchor, constant: 0).isActive = true
                locationInputView.bottomAnchor.constraint(equalTo: blurView.bottomAnchor, constant: 0).isActive = true
                locationInputView.leftAnchor.constraint(equalTo: blurView.leftAnchor, constant: 0).isActive = true
                locationInputView.rightAnchor.constraint(equalTo: blurView.rightAnchor, constant: 0).isActive = true
            }
        }
        else {
            print("No Choices Selected")
        }
    }
    
    func resetSuplyForm() {
        selectedChoices.removeAll()
        if let formController = formVC {
            formController.resetSupplyForm()
        }
    }
    
    func submitClicked(withLocationField locationField:FieldProtocol ) {
        
        if let fieldIndex = self.currentForm!.getIndex(linkName: locationField.linkName){
            self.currentForm!.setField(locationField, at: fieldIndex)
        }
        
        var choicefield = (currentChoiceField as? MultiSelectChoiceFieldProtocol )
        choicefield?.fieldValue.selectedChoices = selectedChoices
        self.currentChoiceField = choicefield
        
        if let choiceField = self.currentChoiceField, let index = self.currentForm?.getIndex(linkName: choiceField.linkName) {
            self.currentForm?.setField(choiceField, at: index)
            ZohoCreatorAPIHandler.submit(form: currentForm!) { (result) in
                switch result{
                case .success(let response):
                    print(response)
                    self.isSupplyViewFormSubmitted = true
                    self.selectedChoices.removeAll()
                    self.locationInputView.configureAfterSuccess(withSelectedSupplies: self.selectedChoices)
                    
                case .failure(let error):
                    print(error)
                }
            }
        }
    }
    
    func cancelClicked() {
        self.formVC?.cancelClicked()
    }
    
}
